/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef DETECTIONBLOBPEAU_H
#define DETECTIONBLOBPEAU_H

/**
 * \author BURGER Brice
 * \since 11/2006
 */

#include "SkinBlobDetector.h"
#include "projection.h"
#include "triangulation.h"
#include "detectionParams.h"


/** 
 * Cette classe utilise les blobs couleur et la triangulation pour extraire des ellipsoides 3D de couleur peau d'une image
 * les distances renvoyés sont en millimètre et les angles en degrés!!
 */
class detectionBlobPeau {

	/* détecteurs 2D */
	SkinBlobDetector *detecteurBlobG;
	SkinBlobDetector *detecteurBlobD;
	
	/* outils de projection */
	projection *projecteur;
	
	int **moyG, **moyD;
	double **covG, **covD, **cov3D;
	int nbBlobG, nbBlobD, nbpt3D;
	int** p3D;
	int* p3d;
	
	
	int nbsuppr, i,j,k, nbpt;
	int ax,ay,az, theta,phi,psi;

	CvMat* Ut,* W,* A;//calcul du SVD
	
	/* ne servent qu'en mode bavard : */
	double alpha;
	CvPoint centre;
	CvSize axes;
	CvRect rect;
	CvMat* Ut2D,* W2D,* A2D;//calcul du SVD en 2D
	
	/* images temporaires pour le calcul */
	IplImage* imgTmpg;
	IplImage* imgTmpd;
	
	/* pour la projection */
	int rayon;
	CvPoint point2dgauche, point2ddroite;
	
	
	public:

	/** constructeur */
	detectionBlobPeau(IplImage* imgG, IplImage* imgD);

	/** destructeur */
	~detectionBlobPeau();
	
	
	/** mise à jour du modèle de couleur
	 * 
	 * \return retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
	 */
	bool MAJModeleCouleur();
	

	/** obtention de l'image de proba peau Gauche dont on s'est servi pour la détection
	 * 
	 */
	IplImage* getImgProbaPeauG();


	/** obtention de l'image de proba peau Droite dont on s'est servi pour la détection
	 * 
	 */
	IplImage* getImgProbaPeauD();
	
	
	/** efface une zone rectangulaire dans les images de proba peau
	 * 
	 */
	void effaceZone(int* _centre, int _rayon);
	
	
	/** Remise À Zéro des images de proba peau
	 * 
	 * cette fonction est necessaire quand on travail sur des images avec ROI :
	 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
	 */
	void RAZ();
	
	
	/** détection des blobs couleurs 3D
	 * 
	 * c'est le détecteur de base, il recherche tous les blobs contenus dans les images données en entrée
	 * limitation : les blobs trop éloignés ou trop proches (au delà de DISTMAXRECHERCHE ou en deçà de DISTMINRECHERCHE) sont éliminés
	 * 
	 * On peut aussi ne rechercher les blobs peau que dans une zone cubique (dont les projections sur les images sont données en entrée)
	 * 
	 * \param bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
	 * \param imgG et imgD sont les images a traiter (gauche et droite)
	 * \param zones : zones 3D de recherche des blobs (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
	 * \param strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
	 * 
	 * \return ellipsoides est la liste des ellipsoides renvoyees
	 * \return on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyees
	 * 
	 * si ellipsoides==NULL, on calcule seulement les images de proba peau (pas de triangulation), on retourne alors -1
	 */
	int detecte(bool bavard, IplImage* imgG, IplImage* imgD, int** ellipsoides, CvRect* zones, bool strict);
	
	
	/** détection des blobs couleurs 3D avec limitations
	 * 
	 * Il s'agit de la même fonction que detecte, mais avec des limitations. La fonction est séparée pour accélérer les calculs.
	 * 
	 * \param bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
	 * \param imgG et imgD sont les images a traiter (gauche et droite)
	 * \param ellipsoides est la liste des ellipsoides renvoyees
	 * \param on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyees
	 * \param strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
	 * 
	 * Limitations possibles :
	 * rectG et rectD : rectangles dans lesquels on ne cherche pas de blobs 2D dans l'image gauche ou droite
	 * pour interdire une zone 3D, il suffit de la projetter et ainsi de mettre l'interdiction sous la forme de d'un rectangle dans chaque image
	 */
	int detecteAvecLimitations(bool bavard, IplImage* imgG, IplImage* imgD, int** ellipsoides, listeRect* rectG, listeRect* rectD, bool strict);
	
	
	
	private:
	
	/*****************************************************************************/
	/***************************** fonctions internes ****************************/
	/*****************************************************************************/

	/** Cette fonction affiche les résultats de la recherche de blobs 2D
	 * 
	 * on affiche les ellipses obtenues à partir des matrices de covariance
	 */
	void parle2D();
	
	/** Cette fonction triangule les blobs 2D, convertie les blobs 3D obtenus en ellipsoïde 3D, puis corrige le résultat
	 * 
	 * \param bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
	 * \param strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
	 * 
	 * \return ellipsoides est la liste des ellipsoides renvoyées
	 * \return on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyées
	 */
	int trianguleEtCorrige(int** ellipsoides, bool bavard, bool strict);
};

#endif
