/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* projection.cpp
 * 
 * Fonctions de projections des objets 3D sur le plan image
 * 
 * Auteur : BURGER Brice
 * Date : 11/2006
 */


#include "projection.h"


/* Constructeur */
projection::projection() {
	
	point3dd = new double[3];
	axesprojd = new double[2];
}


/* Destructeur */
projection::~projection() {
	
	delete[] point3dd;
	delete[] axesprojd;
}


/* Cette fonction projette une ellipsoïde 3D dans le plan
 * particule : paramètres de l'ellipsoïde (3 translations, taille des demi-axes, 3 angles)=(tx, ty, tz, ax, ay, az, alpha, betha, gamma)
 * resultat : paramètres de l'ellipse obtenue par cette projection (angle, taille des demi-axes)=(theta, ax, ay) (grand axe de l'ellipsoïde vers le haut quand theta=0)
 * ATTENTION : le résultat est en mm, comme l'entrée, et non en pixels
 */
void projection::projectionEllipsoide (int* particule, int* resultat) {

	/* particule */
	tx = particule[0];
	ty = particule[1];
	tz = particule[2];
	ax = particule[3];
	ay = particule[4];
	az = particule[5];
	alpha = particule[6]*3.1415/180;//particules en degrés, mais le calcul des cos,sin,... est en radians
	betha = particule[7]*3.1415/180;
	gamma = particule[8]*3.1415/180;
	
	/* matrice de rotation */
	g = cos(betha)*cos(gamma);
	h = -sin(gamma)*cos(alpha) + cos(gamma)*sin(betha)*sin(alpha);
	i = sin(gamma)*sin(alpha) + cos(gamma)*cos(alpha)*sin(betha);
	j = cos(betha)*sin(gamma);
	k = cos(gamma)*cos(alpha) + sin(gamma)*sin(betha)*sin(alpha);
	l = -cos(gamma)*sin(alpha) + sin(gamma)*cos(alpha)*sin(betha);
	m = -sin(betha);
	n = sin(alpha)*cos(betha);
	o = cos(betha)*cos(alpha);
	
	/* translations dans la matrice de passage inversée */
	p = -g*tx - j*ty - m*tz;
	q = -h*tx - k*ty - n*tz;
	r = -i*tx - l*ty - o*tz;
	
	/* on essaye de diminuer le nombre de calculs */
	axcarre = ax*ax;
	aycarre = ay*ay;
	azcarre = az*az;
	
	/* matrice de l'ellipsoïde après changement de repère */
	gp = g*g/axcarre + h*h/aycarre + i*i/azcarre;
	kp = g*j/axcarre + h*k/aycarre + i*l/azcarre;
	op = g*m/axcarre + h*n/aycarre + i*o/azcarre;
	sp = g*p/axcarre + h*q/aycarre + i*r/azcarre;
	
	lp = j*j/axcarre + k*k/aycarre + l*l/azcarre;
	pp = j*m/axcarre + k*n/aycarre + l*o/azcarre;
	tp = j*p/axcarre + k*q/aycarre + l*r/azcarre;
	
	qp = m*m/axcarre + n*n/aycarre + o*o/azcarre;
	up = m*p/axcarre + n*q/aycarre + o*r/azcarre;
	
	vp = p*p/axcarre + q*q/aycarre + r*r/azcarre-1;
	
	/* equation de l'ellipse dans l'espace : ax²+2bxy+cy²+2dx+2ey+f=0 */
	a = sp*sp - gp*vp;
	b = sp*tp - kp*vp;
	c = tp*tp - lp*vp;
	d = sp*up - op*vp;
	e = tp*up - pp*vp;
	f = up*up - qp*vp;
	//si on est en mm, il faut rajouter le facteur d'échelle
	d*=1000;
	e*=1000;
	f*=1000000;
	
	/* on en déduit les paramètres de l'ellipse dont on a besoin */
	if (c-a!=0) theta = atan(2*b/(c-a))/2;
	else theta = 0;//alors c'est un cercle
	
	/* equation de l'ellipse dans l'espace sans rotations : a'x²+c'y²+2d'x+2e'y+f=0 */
	ap = a*cos(theta)*cos(theta) - 2*b*cos(theta)*sin(theta) + c*sin(theta)*sin(theta);
	cp = a*sin(theta)*sin(theta) + 2*b*cos(theta)*sin(theta) + c*cos(theta)*cos(theta);
	dp = d*cos(theta) - e*sin(theta);
	ep = d*sin(theta) + e*cos(theta);
	
	/* on essaye de diminuer le nombre de calculs */
	fpp = dp*dp/ap + ep*ep/cp - f;
	
	/* on en déduit les paramètres de l'ellipse dont on a besoin */
	resultat[0] = int( theta/3.1415*180 );//en degres
	resultat[1] = (int) sqrt(fabs(fpp/ap));//axe suivant x en mm//...ap et cp pourraient éventuellements êtres =0
	resultat[2] = (int) sqrt(fabs(fpp/cp));//axe suivant y en mm
}

/* pour projetter un point 3D sur chaque image en fonction des paramètres intrinsèques */
void projection::projettePoint (int* point3d, CvPoint* point2dgauche, CvPoint* point2ddroite) {

	/* a gauche */
	point2dgauche->x = int (point3d[0]*intrinsicsLeft[2]/point3d[2] + intrinsicsLeft[0]);
	point2dgauche->y = int (point3d[1]*intrinsicsLeft[3]/point3d[2] + intrinsicsLeft[1]);
	
	/* calcul du point 3D vu par la camera de droite */
	point3dd[0] = Mgd[0]*point3d[0] +Mgd[1]*point3d[1] + Mgd[2]*point3d[2] + Mgd[3];
	point3dd[1] = Mgd[4]*point3d[0] +Mgd[5]*point3d[1] + Mgd[6]*point3d[2] + Mgd[7];
	point3dd[2] = Mgd[8]*point3d[0] +Mgd[9]*point3d[1] + Mgd[10]*point3d[2] + Mgd[11];
	
	/* a droite */
	point2ddroite->x = int (point3dd[0]*intrinsicsRight[2]/point3dd[2] + intrinsicsRight[0]);
	point2ddroite->y = int (point3dd[1]*intrinsicsRight[3]/point3dd[2] + intrinsicsRight[1]);
}


/* pour mettre à l'échelle (en pixels) les axes projettés précédemment */
void projection::projetteAxes (int* axesproj, CvSize* axesg, CvSize* axesd) {

	/* à gauche */
	axesg->width = int (axesproj[1]*intrinsicsLeft[2]/1000);//axesproj[1] et [2] parceque le [0] c'est l'angle
	axesg->height = int (axesproj[2]*intrinsicsLeft[3]/1000);
		
	/* calcul des axes vus par la caméra de droite */
	axesprojd[0] = Mgd[0]*axesproj[1] +Mgd[1]*axesproj[2];
	axesprojd[1] = Mgd[4]*axesproj[1] +Mgd[5]*axesproj[2];
	
	/* à droite */
	axesd->width = abs (int (axesprojd[0]*intrinsicsRight[2]/1000));
	axesd->height = abs (int (axesprojd[1]*intrinsicsRight[3]/1000));
}

/* mêmes fonctions de projections de point et mise a l'échelle des axes, mais uniquement du côté gauche */
void projection::projettePointgauche (int* point3d, CvPoint* point2dgauche) {

	point2dgauche->x = int (point3d[0]*intrinsicsLeft[2]/point3d[2] + intrinsicsLeft[0]);
	point2dgauche->y = int (point3d[1]*intrinsicsLeft[3]/point3d[2] + intrinsicsLeft[1]);
}
void projection::projetteAxesgauche (int* axesproj, CvSize* axesg) {

	axesg->width = int (axesproj[1]*intrinsicsLeft[2]/1000);//axesproj[1] et [2] parceque le [0] c'est l'angle
	axesg->height = int (axesproj[2]*intrinsicsLeft[3]/1000);
}

/* même fonction, mais en donnant la matrice intrinsèque soi-même -> permet de projeter dans une autre caméra (ou un autre repère) que celle de GEST */
void projection::projettePointgaucheAutrepart (int* point3d, CvPoint* point2dgauche, int u0, int v0, int au, int av) {

	point2dgauche->x = int (point3d[0]*au/point3d[2] + u0);
	point2dgauche->y = int (point3d[1]*av/point3d[2] + v0);
}



