/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* mesureMvt.cpp
 * 
 * Ce fichier contient tout le nécessaire à une mesure basée sur la couleur :
 * initialisation d'un histogramme de référence
 * calcul de l'histogramme d'une zone (suivant sa forme)
 * calcul de vraissemblance via une distance de Bhattacharrya
 * mise à jour de l'histogramme de référence
 * 
 * Auteur : BURGER Brice
 * Date : 04/2007
 */
#include "mesureCouleur.h"




/* Constructeur */
MesureCouleur::MesureCouleur() {

	incr = 0;
	
	/* Alloc de l'histogramme */
	vectSize = 512*sizeof(double);
	histo = new double[512];
	memset(histo,0,vectSize);

	/* alloc des masques */
	bins_mask = new int[w*h];
	bins_mask_ref = new int[w*h];
	
	bins_tab = new int[256];
	
	/* taille des masques */
	maskSize = w*h*sizeof(int);
	
	/* init des masques */
	for(int i=0;i<w*h;i++) {
		bins_mask[i] = -1;
		bins_mask_ref[i] = -1;
	}

	/* initialisation des tables de conversion */
	InitTablesConv();
}


/* Destructeur */
MesureCouleur::~MesureCouleur() {
	
	delete[] histo;
	
	delete[] bins_mask;
	delete[] bins_mask_ref;
	
	delete[] bins_tab;
	
	if (bins_cub!=NULL) {//ce tableau n'a été créer qu'une seule fois, il ne faut donc le détruire qu'une seule fois
		for (int i=0;i<256;i++) {
			for (int j=0;j<256;j++) delete[] bins_cub[i][j];
			delete[] bins_cub[i];
		}
		delete[] bins_cub;
		bins_cub=NULL;
	}
}

/* Cette fonction initialise les tables de conversion RGB -> bins ,i.e. [255][255][255] -> [512]
 * 
 */
void MesureCouleur::InitTablesConv() {
	
	int ir,ig,ib,r,g,b;
	
	/* Init table de conversion d'une composante (0..255) vers une valeur de bins (0..7) */
	for (int i = 0; i < 256; i++) bins_tab[i] = (int)(((float)i/256.0)*8.0);
	
	if (bins_cub==NULL) {//on n'alloue ce tableau (static) qu'une seule fois car il est très gros (~60Mo)
		/* création du tableau */
		bins_cub = new int**[256];
		for (int i=0;i<256;i++) {
			bins_cub[i] = new int*[256];
			for (int j=0;j<256;j++) bins_cub[i][j] = new int[256];
		}
		/* Init du cube de conversion d'un triplet RGB (0..255 , 0..255 , 0..255) vers une valeur de bins (0..512) avec bin = rbin + 8*gbin + 64*bbin  */
		for (r = 0; r < 256; r++) {
			for (g = 0; g < 256; g++) {
				for (b = 0; b < 256; b++) {
					ir = bins_tab[r];
					ig = bins_tab[g];
					ib = bins_tab[b];
					bins_cub[r][g][b] = (int)(ir + 8*ig + 64*ib);
				}
			}
		}
	}
}


/*! \fn void calcFromRect(int histnum, IplImage* img, CvRect r, double incr)
 *  \brief this method compute color distribution histogram in a rectangle
 *  defined by CvRect structure
 *  \param histnum is the histogram to compute with the ptUL ptLR rectangle
 *  \param img the input RGB image
 *  \param r the englobant rectangle
 */
void MesureCouleur::CalcFromRect(int histnum, IplImage* img, CvRect& r)
{
	register unsigned char * ptr;
	CvPoint ptUL,ptLR;
	ptUL.x = r.x;
	ptUL.y = r.y;
	ptLR.x = r.x+r.width;
	ptLR.y = r.y+r.height;
	
	incr = 1/(r.width*r.height);//on incrémente à chaque fois de 1/surface

	int offsetx = ptUL.x*3;
	int pos,posbin;

	/* RAZ de l'histogramme */
	memset(histo,0,vectSize);

	for(register int i=ptUL.y;i<ptLR.y;i++) {
		ptr = (unsigned char*)img->imageData+i*img->widthStep+offsetx;
		pos = i*img->width;
		
		for(register int j=ptUL.x;j<ptLR.x;j++) {
			posbin=pos+j;
			if(bins_mask[posbin]==-1) bins_mask[posbin] = bins_cub[*(ptr)][*(ptr+1)][*(ptr+2)];
			histo[bins_mask[posbin]]+=incr;
			ptr+=3;
		}
	}
}


/*! \fn void CalcFromEllipse(IplImage* img, CvPoint& centre, CvSize& axes, double angle, double incr)
 *  \brief calcul de l'histogramme de la distribution de couleurs dans une ellipse
 *  \param img : l'image RGB de base
 *  \param centre : coordonnees du centre de l'ellipse
 *  \param axes : tailles des demi-axes de l'ellipse
 *  \param angle : en degres
 */
void MesureCouleur::CalcFromEllipse(IplImage* img, CvPoint& centre, CvSize& axes, int angle) {

	register unsigned char * ptr;
	int posbin;

	double a=axes.width, b=axes.height;
	int maxAxe = int(MAX(a,b));
 
	double c=cos(3.1415*angle/180),s=sin(3.1415*angle/180),Y,A,A4,B,C,inv2A,pDelta,delta; 
	int jmin,jmax,imin,imax;
	
	incr = 1/(3.1415*axes.width*axes.height);//on incrémente à chaque fois de 1/surface

	A = (c*c)/(a*a)+(s*s)/(b*b); 
	B = -2*c*s*(1/(a*a) - 1/(b*b)); 
	C = (s*s)/(a*a)+(c*c)/(b*b); 
   
	pDelta = B*B - A*C*4;
	A4 = A*4;
	inv2A = 1/(2*A);

	imin = MAX(centre.y-maxAxe,0);//pour ne pas sortir de l'image
	imax = MIN(centre.y+maxAxe,h);//pour ne pas sortir de l'image

	/* RAZ de l'histogramme */
	memset(histo,0,vectSize);

	for(register int i=imin;i<imax;i++) {
		Y = i-centre.y;
		delta = pDelta*Y*Y + A4;
		jmin = int( (-B*Y - sqrt(delta))*inv2A + centre.x);
		jmin = MAX(jmin,0);//pour ne pas sortir de l'image
		jmax = int( (-B*Y + sqrt(delta))*inv2A + centre.x);
		jmax = MIN(jmax,w);//pour ne pas sortir de l'image
		ptr = (unsigned char*)img->imageData+i*img->widthStep+jmin*3;
		posbin = i*img->width+jmin;

		for(register int j=jmin;j<jmax;j++) {
			if(bins_mask[posbin]==-1) bins_mask[posbin] = bins_cub[*(ptr)][*(ptr+1)][*(ptr+2)];//Calcul des bins pour chaque couleur
			histo[bins_mask[posbin]]+=incr;
			ptr+=3;
			posbin++;
		}
	}
}


/* calcul de l'histogramme de la distribution de couleurs dans un cercle
 * img : l'image RGB de base
 * centre : coordonnees du centre de l'ellipse
 * rayon : en degres
 */
void MesureCouleur::CalcFromCercle(IplImage* img, CvPoint& centre, int rayon) {

	register unsigned char * ptr;
	int posbin;
 
	double Rcarre, Ycarre;
	int jmin,jmax,imin,imax;

	Rcarre = rayon*rayon;
	
	incr = 1/(3.1415*Rcarre);//on incrémente à chaque fois de 1/surface
	
	imin = MAX(centre.y-rayon,0);//pour ne pas sortir de l'image
	imax = MIN(centre.y+rayon,h);//pour ne pas sortir de l'image	
	
	/* RAZ de l'histogramme */
	memset(histo,0,vectSize);
	
	for(register int i=imin;i<imax;i++) {
		Ycarre = (i-centre.y)*(i-centre.y);
		jmin = int( -sqrt(Rcarre-Ycarre) + centre.x);
		jmin = MAX(jmin,0);//pour ne pas sortir de l'image
		jmax = int( sqrt(Rcarre-Ycarre) + centre.x);
		jmax = MIN(jmax,w);//pour ne pas sortir de l'image
			
		ptr = (unsigned char*)img->imageData+i*img->widthStep+jmin*3;
		posbin = i*img->width+jmin;
	
		for(register int j=jmin;j<jmax;j++) {
			if(bins_mask[posbin]==-1) bins_mask[posbin] = bins_cub[*(ptr)][*(ptr+1)][*(ptr+2)];//Calcul des bins pour chaque couleur
			histo[bins_mask[posbin]]+=incr;
			ptr+=3;
			posbin++;
		}
	}
}


/*! \fn void Display()
 *  \brief this function display the histogram
 */
void MesureCouleur::Disp(string fen, int pause) {

	IplImage *dspimg = cvCreateImage(cvSize(512,256*2),IPL_DEPTH_8U,3);//*2 pour mieux voir
	CvPoint p1,p2;
	int r,g,b;
  
	cvZero(dspimg);
	for(p1.x=0;p1.x<512;p1.x++) {
		p2.x = p1.x+1;
		p1.y = (int)(255*(1-(histo[p1.x]))*2);
		p2.y = 255*2;
      
		b=(int)(p1.x/64);
		g=(int)((p1.x-(64*b))/8);
		r=p1.x-(8*g)-(64*b);
      
		cvRectangle(dspimg,p1,p2,CV_RGB(b*32,g*32,r*32),1);
	}
	cvNamedWindow(fen.c_str(),1);
	cvMoveWindow(fen.c_str(),400,0);
	cvShowImage(fen.c_str(),dspimg);
   
	if(pause) cvWaitKey(0);
}


/* Cette fonction calcule la distance de Bhattacharrya entre l'histogramme calculé précédemment et un histogramme de référence
 * 
 * histref : histogramme de référence
 * deuxsigcarre : sigma de la distance
 */
double MesureCouleur::BhattaDistance(double* histref, float deuxsigcarre) {

	double final=0;

	for(register int i=0;i<512;i++) final+=sqrt(histo[i]*histref[i]);
	double dist = 1.0-final;

	return exp(-(dist/deuxsigcarre));
}


/* Cette fonction met à jour l'histogramme via un histogramme de référence
 * 
 * histref : histogramme par rapport auquel on met à jour
 * alpha : coefficient de mise à jour
 */
void MesureCouleur::MiseAJour(double* histref, double alpha) {

	double unmoinsalpha = 1.0-alpha;
	for(register int i=0;i<512;i++) histo[i] = unmoinsalpha*histo[i]+alpha*histref[i];
}


/* Cette fonction réinitialise le masque */
void MesureCouleur::RAZmasque() {

	/* réinitialisation du masque */
	for(int i=0;i<w*h;i++) bins_mask[i] = -1;
}
