/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef MESURECOULEUR_H
#define MESURECOULEUR_H

/** 
 * 
 * Ce fichier contient tout le nécessaire à une mesure basée sur la couleur :
 * \li initialisation d'un histogramme de référence
 * \li calcul de l'histogramme d'une zone (suivant sa forme)
 * \li calcul de vraissemblance via une distance de Bhattacharrya
 * \li mise à jour de l'histogramme de référence
 * 
 * \author BURGER Brice
 * \since 04/2007
 */

#include <cv.h>
#include <highgui.h>
#include "detectionParams.h"




class MesureCouleur {

   /*! \param int vectSize
    *  \brief this is the vector size in byte
    */
   int vectSize;

   /*! \param int* bins_mask
    *  \brief this is the bins mask used to compute only once each 
    *  RGB to bins
    */
   int* bins_mask;

   /*! \param int* bins_mask_ref
    *  \brief this is the bins mask reference used to initialise the bins_mask
    *  faster
    */
   int* bins_mask_ref;

   /*! \param int maskSize
    *  \brief this is the bins_mask size in byte
    */
   int maskSize;
  
  /*! \param double incr
   *  \brief this is the increment value used to compute a normalized histogram
   *  directly (usable if the number of pixels used to compute histogram is fixed)
   *  incr = 1.0/nbPix
   */
  double incr;


/* pour les tables de conversion */  
int* bins_tab;




public:
static int*** bins_cub;//on déclare ce tableau en static car il est très gros (~60Mo), on ne veux donc le créer qu'une seule fois (dans le main ou le RunTask)
/* histogramme */
double *histo;

/** Constructeur */
MesureCouleur();
  
/** Destructeur */
~MesureCouleur();


/** Cette fonction initialise les tables de conversion RGB -> bins ,i.e. [255][255][255] -> [512]
 * 
 */
void InitTablesConv();



  /*! \fn void InitMask()
   *  \brief this function initialise the bins mask
   */
  inline void InitMask() { memcpy(bins_mask, bins_mask_ref, maskSize); };
  
  
  /*! \fn void calcFromRect(int histnum, IplImage* img, CvRect r, double incr)
   *  \brief this method compute color distribution histogram in a rectangle
   *  defined by CvRect structure
   *  \param histnum is the histogram to compute with the ptUL ptLR rectangle
   *  \param img the input RGB image
   *  \param r the englobant rectangle
   */
  void CalcFromRect(int histnum, IplImage* img, CvRect& r);


  /*! \fn void CalcFromEllipse(IplImage* img, CvPoint& centre, CvSize& axes, double angle, double incr)
   *  \brief calcul de l'histogramme de la distribution de couleurs dans une ellipse
   *  \param img : l'image RGB de base
   *  \param centre : coordonnees du centre de l'ellipse
   *  \param axes : tailles des demi-axes de l'ellipse
   *  \param angle : en degres
   */
  void CalcFromEllipse(IplImage* img, CvPoint& centre, CvSize& axes, int angle);
  
/* calcul de l'histogramme de la distribution de couleurs dans un cercle
 * img : l'image RGB de base
 * centre : coordonnees du centre de l'ellipse
 * rayon : en degres
 */
void CalcFromCercle(IplImage* img, CvPoint& centre, int rayon);


  /*! \fn void Display()
   *  \brief this function display the histogram
   */
  void Disp(string fen, int pause);


/** Cette fonction calcule la distance de Bhattacharrya entre l'histogramme calculé précédemment et un histogramme de référence
 * 
 * \param histref : histogramme de référence
 * \param deuxsigcarre : sigma de la distance
 */
double BhattaDistance(double* histref, float deuxsigcarre);


/** Cette fonction met à jour l'histogramme via un histogramme de référence
 * 
 * \param histref : histogramme par rapport auquel on met à jour
 * \param alpha : coefficient de mise à jour
 */
void MiseAJour(double* histref, double alpha);


/** Cette fonction réinitialise le masque */
void RAZmasque();



};

#endif
