/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* traqueurDeDeuxMains.cpp
 * 
 * Programme principal, il fait l'interface entre le main ou genom et le reste du code.
 * Ici sont implémentés les fonctions de haut niveau du traqueur 3D à filtre particulaire.
 * Le but de ce programme est de traquer les deux mains d'un bonhomme dans l'espace (forme ellipsoidale) à partir des images d'un banc stéréo.
 * 
 * L'affichage se fait avec l'image gauche en haut (vue du robot).
 * 
 * CODE COULEUR DE L'AFFICHAGE : (RGB)
 * 
 * détections -> rouge (100)
 * tête -> bleu clair (011) à blanc (111)
 * main gauche -> vert (010) à jaune (110)
 * main droite -> bleu (001) à violet (101)
 * 
 * 
 * Auteur : BURGER Brice
 * Date : 02/2007
 */


#include "traqueurDeDeuxMains.h"

static bool BAVARD = false;//s'il doit être bavard durant la traque//...faudra faire ça plus proprement avec une fonction message()


/**************************************/
/* fonctions générales :              */
/**************************************/

/* constructeur du traqueur 3D :
 * 
 * imgG et imgD : images représentatives de celles qui serviront ensuite durant la traque (même taille, même convention de couleur, ...)
 */
Traqueur2::Traqueur2(IplImage* imgG, IplImage* imgD) {

	/* taille de l'image */
	w = imgG->width;
	h = imgG->height;
	
	/* création des particules */
	nbcibles = 3;
	cibles = new Particules*[nbcibles];
	cibles[TETE] = new Particules(nbvarSphere);
	cibles[MAINgAUCHE] = new Particules(nbvarEllipsoide);
	cibles[MAINdROITE] = new Particules(nbvarEllipsoide);
	particulesTETE = cibles[TETE];
	particulesMG = cibles[MAINgAUCHE];
	particulesMD = cibles[MAINdROITE];

	nbvar = 0;
	for (int i=0;i<nbcibles;i++) nbvar += cibles[i]->nbvar;//nombre de variables du vecteur d'état
	
	VEGlobal = new int[nbvar];//vecteur d'état global
	
	/* initialisation des espaces de travail des fonctions utilisées */
	initTriangulation();
	initIDMOT(nbcibles);
	
	/* création des détecteurs */
	detecteurBlob = new detectionBlobPeau(imgG,imgD);
	detecteurBlobTETE = new detectionBlobPeau(imgG,imgD);//on crée un détecteur à part pour la tête (pour pouvoir la supprimer)
	detecteurVisage = new detectionVisage();
	
	/* création des mesures */
	mesureMvt = new MesureMvt3D();
	mesureDistance = new MesureDistance3D();
	
	/* outils */
	proj = new projection();
	eval2D = new evaluation();

	projections = new int[3];
	
	g = new double*[nbcibles];//...g, suivi, zones, zones3D, centres et niveau_reinit devraient peut-être être déplacés vers particules, mais c'est pas sûr (ça risque d'être moins partique pour l'accès surtout avec les for)
	for (int i=0;i<nbcibles;i++) g[i] = new double[nbpart];
	gTETE = g[TETE];
	gMG = g[MAINgAUCHE];
	gMD = g[MAINdROITE];
	gmax = new double[nbcibles];
	gmaxTETE = &(gmax[TETE]);
	gmaxMG = &(gmax[MAINgAUCHE]);
	gmaxMD = &(gmax[MAINdROITE]);
	evalFinale = new double[nbcibles];
	
	suivi = new bool[nbcibles];
	T = &(suivi[TETE]);
	MG = &(suivi[MAINgAUCHE]);
	MD = &(suivi[MAINdROITE]);
	
	imgSup = cvCreateImage(cvSize(w,h),8,3);
	
	CheminSauvegarde = new char[128];//pour sauvegarder des images
	strcpy(CheminSauvegarde,CHEMIN_SAUV_DEFAUT);
	nomsauvegarde = new char[128];//pour sauvegarder des images
	
	/* images NG pour pouvoir afficher l'image de proba peau */
	for (int i=0;i<3;i++) imageNG[i] = cvCreateImage(cvSize(w,h),8,1);
	
	AVEC_INTERFACE = false;
	AFFICHE_SUP = false;
	AFFICHE_SUP_GAUCHE = false;
	AFFICHE_PROBA_PEAU = false;
	AFFICHE_MVT = false;
	AFFICHE_DIST = false;
	AFFICHE_PARTICULES = false;
	SAUVEGARDE_IMAGES = false;
	SAUVEGARDE_RESULTATS = false;
	fichierResultats = NULL;
	
	/* zones de recherche */
	zones = new CvRect*[nbcibles];
	zones3D = new int*[nbcibles];
	centres = new int*[nbcibles];
	for (int i=0;i<nbcibles;i++) {
		zones[i] = new CvRect[2];
		zones3D[i] = new int[3];
		centres[i] = new int[3];
	}
	niveau_reinit = new int[nbcibles];//niveau de la reinitialisation pour chaque cible de 0 (pas de reinitialisation) à NIVEAU_REINIT_MAX (zone maximum = celle d'initialisation)
	
	numero=0;
}


/* destructeur du traqueur 3D :
 *
 */
Traqueur2::~Traqueur2() {
	
	avec_interface(false);
	
	for (int i=0;i<nbcibles;i++) {
		delete cibles[i];
		delete[] g[i];
		delete[] zones[i];
		delete[] zones3D[i];
		delete[] centres[i];
	}
	delete[] cibles;
	delete[] g;
	delete[] zones;
	delete[] zones3D;
	delete[] centres;
	
	delete[] gmax;
	delete[] suivi;
	delete[] niveau_reinit;
	delete[] evalFinale;
	
	delete[] VEGlobal;
	
	delete[] projections;
	
	cvReleaseImage(&imgSup);
	for (int i=0;i<3;i++) cvReleaseImage(&imageNG[i]);
	
	delete[] CheminSauvegarde;
	delete[] nomsauvegarde;
	
	if (fichierResultats) {
		fclose(fichierResultats);
		fichierResultats = NULL;
	}
	
	/* on relache les espaces de travail des fonctions utilisées */
	finTriangulation();
	finIDMOT(nbcibles);
	
	delete detecteurBlob;
	delete detecteurBlobTETE;
	delete detecteurVisage;
	
	delete mesureMvt;
	delete mesureDistance;
	
	/* outils */
	delete proj;
	delete eval2D;
}


/* changement d'espace de couleur
 * 
 * retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
 */
bool Traqueur2::ChangeEspaceCouleur(int EspaceCoul) {
	
	espaceCoul = EspaceCoul;
	
	if (!detecteurBlobTETE->MAJModeleCouleur()) return false;
	if (!detecteurBlob->MAJModeleCouleur()) return false;
	
	return true;
}


/* fonction d'initialisation globale du traqueur : (il peut démarrer avec la tête seule)
 * 
 * imgG,imgD : images pour l'affichage (gauche et droite)
 * imgTG,imgTD : images de travail (gauche et droite)
 * les images d'affichage et de travail peuvent être les mêmes
 * 
 * Ici on initialise à partir d'une détection de tête
 * ATTENTION : il ne doit y avoir qu'un seul visage !! Sinon c'est le premier qui est pris.
 * 
 * retourne l'état du système (via l'énumération ETAT_GEST) :
 * 	RIEN=0,			//rien n'est traqué
 * 	TOUT=1,			//tout est traqué
 * 	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
 * 	PASmAINdROITE=3,	//tout est traqué sauf la main droite
 * 	AUCUNEmAIN=4,		//seule la tête est traquée
 * 	PAStETE=5,		//tout est traqué sauf la tête
 * 	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
 * 	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
 */
ETAT_GEST Traqueur2::INIT(IplImage* imgG,IplImage* imgD , IplImage* imgTG,IplImage* imgTD) {
	
	if (BAVARD) cout << "\nINITIALISATION sur les deux mains et la tete" << endl;
	int i,j;
	
	particulesTETE->RAZinterdictions();
	particulesMG->RAZinterdictions();
	particulesMD->RAZinterdictions();
	for (i=0;i<nbcibles;i++) niveau_reinit[i] = NIVEAU_REINIT_MAX;//RAZ de niveau_reinit
	ETAT = RIEN;
	
	/* on commence par une détection de blobs peau sur toute l'image *///comme on calcule ici pour toute l'image, les détections ne seront stoquée que du coté gauche (MG)
	particulesMG->nbpartdetectees = detecteurBlob->detecte(BAVARD, imgTG, imgTD, particulesMG->particuleDetectees,NULL, false);//on n'est pas strict ici parcequ'on veux pouvoir détecter une tete (qui est bien plus grande qu'une main)
	
	/* préparation de l'affichage de l'image de proba peau */
	if (AFFICHE_SUP) {
		if (AFFICHE_PROBA_PEAU) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(detecteurBlob->getImgProbaPeauG(), imageNG[0]);
			else cvCopyImage(detecteurBlob->getImgProbaPeauD(), imageNG[0]);
		}
		if (AFFICHE_MVT) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(mesureMvt->getImgMvtG(), imageNG[0]);
			else cvCopyImage(mesureMvt->getImgMvtD(), imageNG[0]);
		}
		if (AFFICHE_DIST) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(mesureDistance->getImgDistG(), imageNG[0]);
			else cvCopyImage(mesureDistance->getImgDistD(), imageNG[0]);
		}
		cvCopyImage(imageNG[0], imageNG[1]);
		cvCopyImage(imageNG[0], imageNG[2]);
		cvMerge(imageNG[0],imageNG[1],imageNG[2],NULL,imgSup);
	}
	
	/* si aucun blob n'est trouvé, inutile de continuer */
	if (particulesMG->nbpartdetectees==0) {
		if (BAVARD) cout << "aucun blob 3D detecte" << endl;
		if (AVEC_INTERFACE) {
			cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			if (AFFICHE_SUP) affiche(imgG, imgD, imgSup);
			else affiche(imgG, imgD);
		}
		return RIEN;//si on ne détecte rien
	}
	particulesMG->invnbpartdetectees = 1/(double)particulesMG->nbpartdetectees;
	
	/* on recherche des visages autour des blobs */
	particulesTETE->nbpartdetectees = 0;
	for (i=0 ; i<particulesMG->nbpartdetectees ; i++) {//pour chaque blob détecté
		/* création des zones de recherche */
		rayon = 1.2*RayonTete;//rayon de recherche = rayon de la tête + 20% pour ne pas être trop juste pour le détecteur de visage
		rect = zones[TETE];
		proj->projettePoint (particulesMG->particuleDetectees[i], &point2dgauche, &point2ddroite);//projection du centre
		L = int (rayon*intrinsicsLeft[2]/particulesMG->particuleDetectees[i][2]);//rayon suivant x
		H = int (rayon*intrinsicsLeft[3]/particulesMG->particuleDetectees[i][2]);//rayon suivant y
		rect[0].x = point2dgauche.x-L;
		rect[0].y = point2dgauche.y-H;
		rect[0].width = L*2;
		rect[0].height = H*2;
		L = int (rayon*intrinsicsRight[2]/particulesMG->particuleDetectees[i][2]);//rayon suivant x
		H = int (rayon*intrinsicsRight[3]/particulesMG->particuleDetectees[i][2]);//rayon suivant y
		rect[1].x = point2ddroite.x-L;
		rect[1].y = point2ddroite.y-H;
		rect[1].width = L*2;
		rect[1].height = H*2;
		/* vérifications (ce que opencv ne gère pas) */
		for (j=0;j<2;j++) verifRectOpencv(rect[j]);
		/* détection */
		if ( detecteurVisage->detecteUnSeul(imgTG, imgTD, particulesTETE->particuleDetectees[particulesTETE->nbpartdetectees], rect) ) particulesTETE->nbpartdetectees++;
		/* pour voir où on cherche le visage */
		if (AFFICHE_SUP) {
			if (AFFICHE_SUP_GAUCHE) j=0;
			else j=1;
			cvRectangle(imgSup, cvPoint(rect[j].x, rect[j].y), cvPoint(rect[j].x + rect[j].width, rect[j].y + rect[j].height), CV_RGB(255,255,255),2);
		}
	}
	
	/* init de la tête */
	if (particulesTETE->nbpartdetectees==1) memcpy(particulesTETE->particuleFinale, particulesTETE->particuleDetectees[0], particulesTETE->nbvar*sizeof(int));
	if (particulesTETE->nbpartdetectees>1) {//plusieurs détections : on choisi le visage le plus proche
		int dist = 10000;//s'il est à plus de 10m, c'est de toutes façons qu'il y a un problème
		int itmp=-1;
		for (i=0; i<particulesTETE->nbpartdetectees; i++) if (particulesTETE->particuleDetectees[0][2]<dist) {//on cherche le plus proche
			dist = particulesTETE->particuleDetectees[0][2];
			itmp = i;
		}
		if (itmp==-1) particulesTETE->nbpartdetectees=0;//finalement rien de bon
		else {
			particulesTETE->nbpartdetectees=1;
			memcpy(particulesTETE->particuleFinale, particulesTETE->particuleDetectees[itmp], particulesTETE->nbvar*sizeof(int));
		}
	}
	if (particulesTETE->nbpartdetectees==0) {//aucun détection : on arrête là
		if (BAVARD) cout << "Aucun visage detecte" << endl;
		if (AVEC_INTERFACE) {
			cvLine( imgG, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			cvLine( imgD, cvPoint(0,0), cvPoint(w,h), CV_RGB(0,0,0),4);
			if (AFFICHE_SUP) affiche(imgG, imgD, imgSup);
			else affiche(imgG, imgD);
		}
		return RIEN;//si on ne détecte rien
	}
	
	/* la particule initiale est l'équivalent d'une particule parfaite (poids 1, aucune autre particule conservée) */
	memcpy(particulesTETE->particuleFinalePrec,particulesTETE->particuleFinale,particulesTETE->nbvar*sizeof(int));//tête
	memcpy(particulesTETE->particulesPrec[0],particulesTETE->particuleFinale,particulesTETE->nbvar*sizeof(int));//tête
	particulesTETE->nbpartconservees = 1;
	particulesTETE->invnbpartdetectees = 1;
	particulesTETE->poidsPrec[0] = 1;
	*T = true;
	niveau_reinit[TETE] = 0;
	ETAT = AUCUNEmAIN;
	
	/* init de la main gauche : */
	//on interdit la tête
	particulesMG->interdire2D(particulesTETE->particuleFinale, RayonInterdictionT);//à l'init on considère que les mains ne peuvent pas être devant la tête (quelque soit z)
	calculeZoneMax(MAINgAUCHE);
	*MG = reinit(MAINgAUCHE);
	if (*MG) {
		niveau_reinit[MAINgAUCHE] = 0;
		ETAT = PASmAINdROITE;
	}
	
	/* init de la main droite : */
	//les blobs détectés sont les mêmes que pour la main gauche
	particulesMD->nbpartdetectees = particulesMG->nbpartdetectees;
	particulesMD->invnbpartdetectees = particulesMG->invnbpartdetectees;
	for (i=0;i<particulesMG->nbpartdetectees;i++) memcpy(particulesMD->particuleDetectees[i], particulesMG->particuleDetectees[i], particulesMD->nbvar*sizeof(int));
	//on interdit la tête et la MG si elle a été trouvée
	particulesMD->interdire2D(particulesTETE->particuleFinale, RayonInterdictionT);//à l'init on considère que les mains ne peuvent pas être devant la tête (quelque soit z)
	if (*MG) particulesMD->interdire(particulesMG->particuleFinale, RayonInterdictionM);
	calculeZoneMax(MAINdROITE);
	*MD = reinit(MAINdROITE);
	if (*MD) {
		niveau_reinit[MAINdROITE] = 0;
		if (*MG) ETAT = TOUT;
		else ETAT = PASmAINgAUCHE;
	}
	
	/* initialisation de la mesure de mouvement */
	mesureMvt->calculeImgMvt(true, imgTG, imgTD, NULL);
	
	/* pour l'histogramme de couleur de la tête (initialisation) */
	proj->projettePoint (particulesTETE->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
	int rayon = int (RayonTete*intrinsicsLeft[2]/particulesTETE->particuleFinale[2]);//rayon du cercle pour l'affichage de la tête et pour le calcul des couleurs
	eval2D->init(imgTG, point2dgauche, rayon);
	
	/* visualisation finale */
	if (AVEC_INTERFACE) {
		
		/************** on reprojette la tête sur les images pour visualiser la particule finale *************/
		temp = particulesTETE->particuleFinale;
		
		cvCircle( imgG, point2dgauche, rayon, CV_RGB(255,255,255), 2 );//pour simplifier, on néglige les déformations de l'image et on affiche un cercle
		cvCircle( imgD, point2ddroite, rayon, CV_RGB(255,255,255), 2 );
		
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule gauche finale *************/
		if (*MG) {
			temp = particulesMG->particuleFinale;

			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,255,0),2);
			cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,255,0),2);
		}
				
		/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule droite finale *************/
		if (*MD) {
			temp = particulesMD->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,255),2);
			cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,255),2);
		}
		
		/************** affichage **************/
		if (AFFICHE_SUP) affiche(imgG, imgD, imgSup);
		else affiche(imgG, imgD);
	}
	
	/* RAZs */
	detecteurBlob->RAZ();//pour éviter les problèmes de particules allant dans des zones non calculées
	detecteurBlobTETE->RAZ();//pour éviter les problèmes de particules allant dans des zones non calculées
	particulesTETE->RAZinterdictions();
	particulesMG->RAZinterdictions();
	particulesMD->RAZinterdictions();
	
	if (BAVARD) cout << "init faite : ON PEUT COMMENCER" << endl;
	
	return ETAT;
}


/* pour avancer d'un pas dans le filtre particulaire :
 * 
 * imgG,imgD : images pour l'affichage (gauche et droite)
 * imgTG,imgTD : images de travail (gauche et droite)
 * les images d'affichage et de travail peuvent être les mêmes
 * 
 * retourne l'état du système (via l'énumération ETAT_GEST) :
 * 	RIEN=0,			//rien n'est traqué
 * 	TOUT=1,			//tout est traqué
 * 	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
 * 	PASmAINdROITE=3,	//tout est traqué sauf la main droite
 * 	AUCUNEmAIN=4,		//seule la tête est traquée
 * 	PAStETE=5,		//tout est traqué sauf la tête
 * 	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
 * 	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
 */
ETAT_GEST Traqueur2::PAS(IplImage* imgG, IplImage* imgD , IplImage* imgTG,IplImage* imgTD) {
	
	register int i;
	register int j;
	*gmaxTETE = 0;
	*gmaxMG = 0;
	*gmaxMD = 0;
	bool AFFICHE_DIFF_IDMOT = false;
	if (AFFICHE_PARTICULES) AFFICHE_DIFF_IDMOT = true;
	bool debug = false;//...à virer
	
	if (BAVARD) printf("\nnouveau PAS :\n");
	
	/**** 0) utilisation d'un algo de type IDMOT (préambule) ****/
	preambuleIDMOT(cibles, nbcibles, suivi);
	
	
	/**** 1) création des zones de recherche ****/
	for (i=0;i<nbcibles;i++) {//pour chaque cibles
		
		/* zone 3D (dépend du niveau de réinit) */
		if (niveau_reinit[i]==NIVEAU_REINIT_MAX) {//zones maximales, et d'initialisation
			calculeZoneMax((TYPE_PARTICULE_GEST)i);
			proj->projettePoint (centres[i], &point2dgauche, &point2ddroite);//projection du centre
			L = int (zones3D[i][0]*intrinsicsLeft[2]/centres[i][2]);
			H = int (zones3D[i][1]*intrinsicsLeft[3]/centres[i][2]);
		} else {//zones cubiques centrées sur la particule finale précédente
			particuleTemp = cibles[i];
			if (i==TETE) rayon = (niveau_reinit[i]+1) * VARDynsphere[1]*3;	//rayon = niveau de réinit * distance maximale que la cible a pu parcourir en 1 tours (on peut considérer que 3 * la dynamique est une bonne approximation)
			else rayon = (niveau_reinit[i]+1) * VARDynellipsoide[1]*3;	//on part ici également du principe que la dynamique est la même suivant x et y
			zones3D[i][0] = rayon;
			zones3D[i][1] = rayon;
			zones3D[i][2] = rayon;
			proj->projettePoint (particuleTemp->particuleFinale, &point2dgauche, &point2ddroite);//projection du centre
			L = int (rayon*intrinsicsLeft[2]/particuleTemp->particuleFinale[2]);
			H = int (rayon*intrinsicsLeft[3]/particuleTemp->particuleFinale[2]);
		}
		
		/* zone 2D */
		rect = zones[i];
		rect[0].x = point2dgauche.x-L;
		rect[0].y = point2dgauche.y-H;
		rect[0].width = L*2;
		rect[0].height = H*2;
		rect[1].x = point2ddroite.x-L;//il faudrait recalculer L et H avec les intrinsicsRight[2]et[3], mais la différence est généralement négligeable
		rect[1].y = point2ddroite.y-H;
		rect[1].width = L*2;
		rect[1].height = H*2;
		
		/* vérifications (ce que opencv ne gère pas) */
		for (j=0;j<2;j++) verifRectOpencv(rect[j]);
	}
	
	
	/**** 2) détections et initialisation des mesures ****/
	/* détection des blobs peau */
	if (BAVARD) printf("**** mains gauche :\n");
	particulesMG->nbpartdetectees = detecteurBlob->detecte(BAVARD, imgTG, imgTD, particulesMG->particuleDetectees, zones[MAINgAUCHE], true);
	if (BAVARD) printf("**** mains droite :\n");
	particulesMD->nbpartdetectees = detecteurBlob->detecte(BAVARD, imgTG, imgTD, particulesMD->particuleDetectees, zones[MAINdROITE], true);
	if (BAVARD) printf("**** tête :\n");
	particulesTETE->nbpartdetectees = detecteurBlobTETE->detecte(BAVARD, imgTG, imgTD, NULL, zones[TETE], false);//pour calculer l'image de proba peau (nécessaire à l'évaluation des visages)
	if (BAVARD) printf("**** fin\n");
	if (particulesMG->nbpartdetectees!=0) particulesMG->invnbpartdetectees = 1/(double)particulesMG->nbpartdetectees;
	if (particulesMD->nbpartdetectees!=0) particulesMD->invnbpartdetectees = 1/(double)particulesMD->nbpartdetectees;
	
	/* détection de visages dans les environs de la détection de visage précédente (on ne traque qu'un seul visage, donc les autres ne nous intéressent pas) */
	if (particulesTETE->nbpartdetectees!=0) {//pour avoir moins de fausses détections de visage, on oblige un visage à être sur un blob peau.
		particulesTETE->nbpartdetectees = detecteurVisage->detecteAvecZones(imgTG,imgTD,particulesTETE->particuleDetectees,zones[TETE]);
		if (particulesTETE->nbpartdetectees!=0) {
			particulesTETE->invnbpartdetectees = 1/particulesTETE->nbpartdetectees;
			if (BAVARD) cout << particulesTETE->nbpartdetectees << "visage détecté !" << endl;
			for (i=0;i<particulesTETE->nbpartdetectees;i++) detecteurBlob->effaceZone(particulesTETE->particuleDetectees[i], RayonTete);//on supprime le visage des détections des mains//...c'est ad hoc !
		} else if (BAVARD) cout << "aucun visage détecté !" << endl;
	} else {
		particulesTETE->nbpartdetectees = 0;
		if (BAVARD) cout << "aucun blob pour le visage détecté !" << endl;
	}
	
	/* initialisations des mesures */
	//de mouvement dans les zones de calculs (pas la peine pour la tête)
	mesureMvt->calculeImgMvt(true, imgTG, imgTD, zones[MAINgAUCHE]);
	mesureMvt->calculeImgMvt(false, imgTG, imgTD, zones[MAINdROITE]);
	//de distance
	if (CalculeImgDist) {
		mesureDistance->calculeImgDist(imgTG, imgTD, zones[TETE]);
		mesureDistance->calculeImgDist(imgTG, imgTD, zones[MAINgAUCHE]);
		mesureDistance->calculeImgDist(imgTG, imgTD, zones[MAINdROITE]);
	}

	if (AFFICHE_SUP) {
		if (AFFICHE_PROBA_PEAU) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(detecteurBlob->getImgProbaPeauG(), imageNG[0]);
			else cvCopyImage(detecteurBlob->getImgProbaPeauD(), imageNG[0]);
		}
		if (AFFICHE_MVT) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(mesureMvt->getImgMvtG(), imageNG[0]);
			else cvCopyImage(mesureMvt->getImgMvtD(), imageNG[0]);
		}
		if (AFFICHE_DIST) {
			if (AFFICHE_SUP_GAUCHE) cvCopyImage(mesureDistance->getImgDistG(), imageNG[0]);
			else cvCopyImage(mesureDistance->getImgDistD(), imageNG[0]);
		}
		cvCopyImage(imageNG[0], imageNG[1]);
		cvCopyImage(imageNG[0], imageNG[2]);
		cvMerge(imageNG[0],imageNG[1],imageNG[2],NULL,imgSup);
		/* affichage des zones de recherche */
		if (AFFICHE_SUP_GAUCHE) j=0;
		else j=1;
		cvRectangle(imgSup, cvPoint(zones[TETE][j].x, zones[TETE][j].y), cvPoint(zones[TETE][j].x + zones[TETE][j].width, zones[TETE][j].y + zones[TETE][j].height), CV_RGB(255,255,255),2);
		cvRectangle(imgSup, cvPoint(zones[MAINgAUCHE][j].x, zones[MAINgAUCHE][j].y), cvPoint(zones[MAINgAUCHE][j].x + zones[MAINgAUCHE][j].width, zones[MAINgAUCHE][j].y + zones[MAINgAUCHE][j].height), CV_RGB(255,255,0),2);
		cvRectangle(imgSup, cvPoint(zones[MAINdROITE][j].x, zones[MAINdROITE][j].y), cvPoint(zones[MAINdROITE][j].x + zones[MAINdROITE][j].width, zones[MAINdROITE][j].y + zones[MAINdROITE][j].height), CV_RGB(255,0,255),2);
	}
	
	
	/**** 3) génération, évaluation et selection des particules ****/
	/* pour chaque particule : 
	 * 
	 * 1) Prédiction (génération des nouvelles particules)
	 * 2) Calcul de la vraisemblance
	 * 
	 * si une cible n'est plus suivie, elle ne peut-être remise en place que par réinitialisation douce
	 */
	for(i=0;i<nbpart;i++) {
		
		/* tête */
		if (*T) {
			lamdaTETE = particulesTETE->genere(i);//Prédiction
			if (lamdaTETE!=0) gTETE[i] = eval(particulesTETE->particules[i], imgTG, imgTD, TETE);//calcul de la vraisemblance
			gTETE[i] *= lamdaTETE;
			if (*gmaxTETE<gTETE[i]) *gmaxTETE=gTETE[i];
		}
		/* main gauche */
		if (*MG) {
			lamdaMG = particulesMG->genere(i);//Prédiction
			if (lamdaMG!=0) gMG[i] = eval(particulesMG->particules[i], imgTG, imgTD, MAINgAUCHE);//calcul de la vraisemblance
			gMG[i] *= lamdaMG;
			if (*gmaxMG<gMG[i]) *gmaxMG=gMG[i];
			
		}
		/* main droite */
		if (*MD) {
			lamdaMD = particulesMD->genere(i);//Prédiction
			if (lamdaMD!=0) gMD[i] = eval(particulesMD->particules[i], imgTG, imgTD, MAINdROITE);//calcul de la vraisemblance
			gMD[i] *= lamdaMD;
			if (*gmaxMD<gMD[i]) *gmaxMD=gMD[i];
		}
	}
	
	/* on détermine si les particules sont encore suivies */
	if (*gmaxTETE==0) *T = false;
	else *T = true;
	if (*gmaxMG==0) *MG = false;
	else *MG = true;
	if (*gmaxMD==0) *MD = false;
	else *MD = true;
	
	/* calcul du poids */
	for(i=0;i<nbpart;i++) {
		if (*T) particulesTETE->poids[i] = eval2D->poids(gTETE[i],*gmaxTETE);
		if (*MG) particulesMG->poids[i] = eval2D->poids(gMG[i],*gmaxMG);
		if (*MD) particulesMD->poids[i] = eval2D->poids(gMD[i],*gmaxMD);
	}
	
	int pTETE=0, pMG=0, pMD=0;//nombre de particules de poids non nul à la fin du tour//...à virer
	if (debug) {//...à virer (tout le paragraphe)
		/* décompte des particules ayant un poids non nul */
		for(i=0;i<nbpart;i++) {
			if (*T) if (particulesTETE->poids[i]!=0) pTETE++;
			if (*MG) if (particulesMG->poids[i]!=0) pMG++;
			if (*MD) if (particulesMD->poids[i]!=0) pMD++;
		}
		cout << "nb particules non nulles a la fin du tour avant IDMOT : TETE(" << pTETE << ") , MG(" << pMG << ") , MD(" << pMD << ")" << endl;
	}
	
	/* sélection de la particule finale */
	if (*T) *T = particulesTETE->selectionne();
	if (*MG) *MG = particulesMG->selectionne();
	if (*MD) *MD = particulesMD->selectionne();

	/* affichage des particules ayant un poids non nul *///en fait ce seront les particules éliminées par IDMOT
	if (AFFICHE_DIFF_IDMOT) {
		for(i=0;i<nbpart;i++) {
			if (*T) if (particulesTETE->poids[i]!=0) {
				proj->projettePoint (particulesTETE->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				if (AFFICHE_SUP_GAUCHE) cvCircle(imgSup, point2dgauche, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particules[i][2]), CV_RGB((int)(255*particulesTETE->poids[i]),255,255), 2 );
				else cvCircle(imgSup, point2ddroite, int (RayonTete*intrinsicsRight[2]/particulesTETE->particules[i][2]), CV_RGB(255,0,0), 2 );
			}
			if (*MG) if (particulesMG->poids[i]!=0) {
				proj->projettePoint (particulesMG->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				proj->projectionEllipsoide (particulesMG->particules[i], projections);
				proj->projetteAxes (projections, &axesg, &axesd);
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),255,0),2);
				else cvEllipse(imgSup,point2ddroite,axesd,projections[0],0,360,CV_RGB(100,100,100),2);
			}
			if (*MD) if (particulesMD->poids[i]!=0) {
				proj->projettePoint (particulesMD->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				proj->projectionEllipsoide (particulesMD->particules[i], projections);
				proj->projetteAxes (projections, &axesg, &axesd);
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),0,255),2);
				else cvEllipse(imgSup,point2ddroite,axesd,projections[0],0,360,CV_RGB(255,255,255),2);
			}
		}
		
		int rayon;//rayon du cercle pour l'affichage de la tête
		
		if (*T) {
			/************** on reprojette la tête sur les images pour visualiser la particule finale *************/
			temp = particulesTETE->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			rayon = int (RayonTete*intrinsicsLeft[2]/temp[2]);
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvCircle(imgSup,point2dgauche,rayon,CV_RGB(100,100,100),2);
				else cvCircle(imgSup,point2ddroite,rayon,CV_RGB(100,100,100),2);
			}
		}
		
		if (*MG) {
			/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule gauche finale *************/
			temp = particulesMG->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,0),2);
				else cvEllipse(imgSup,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,0),2);
			}
		}
		
		if (*MD) {
			/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule droite finale *************/
			temp = particulesMD->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,angleg,0,360,CV_RGB(130,0,0),2);
				else cvEllipse(imgSup,point2ddroite,axesd,angled,0,360,CV_RGB(130,0,0),2);
			}
		}
	}

	/* utilisation d'un algo de type IDMOT */
	IDMOT(cibles,nbcibles,suivi,g,gmax);

	/* affichage des particules ayant un poids non nul */
	if (AFFICHE_PARTICULES) {
		for(i=0;i<nbpart;i++) {
			if (*T) if (particulesTETE->poids[i]!=0) {
				proj->projettePoint (particulesTETE->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				if (AFFICHE_SUP_GAUCHE) cvCircle(imgSup, point2dgauche, int (RayonTete*intrinsicsLeft[2]/particulesTETE->particules[i][2]), CV_RGB((int)(255*particulesTETE->poids[i]),255,255), 2 );
				else cvCircle(imgSup, point2ddroite, int (RayonTete*intrinsicsRight[2]/particulesTETE->particules[i][2]), CV_RGB((int)(255*particulesTETE->poids[i]),255,255), 2 );
			}
			if (*MG) if (particulesMG->poids[i]!=0) {
				proj->projettePoint (particulesMG->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				proj->projectionEllipsoide (particulesMG->particules[i], projections);
				proj->projetteAxes (projections, &axesg, &axesd);
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),255,0),2);
				else cvEllipse(imgSup,point2ddroite,axesd,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),255,0),2);
			}
			if (*MD) if (particulesMD->poids[i]!=0) {
				proj->projettePoint (particulesMD->particules[i], &point2dgauche, &point2ddroite);//projection du centre
				proj->projectionEllipsoide (particulesMD->particules[i], projections);
				proj->projetteAxes (projections, &axesg, &axesd);
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),0,255),2);
				else cvEllipse(imgSup,point2ddroite,axesd,projections[0],0,360,CV_RGB((int)(255*particulesMG->poids[i]),0,255),2);
			}
		}
	}
	
	if (debug) {//...à virer (tout le paragraphe)
		pTETE=0;
		pMG=0;
		pMD=0;
		/* décompte des particules ayant un poids non nul *///...
		for(i=0;i<nbpart;i++) {
			if (*T) if (particulesTETE->poids[i]!=0) pTETE++;
			if (*MG) if (particulesMG->poids[i]!=0) pMG++;
			if (*MD) if (particulesMD->poids[i]!=0) pMD++;
		}
		cout << "nb particules non nulles a la fin du tour : TETE(" << pTETE << ") , MG(" << pMG << ") , MD(" << pMD << ")" << endl;//...
	}
	
	/* une main ne doit pas se trouver à plus de 1m de la tête */
	if (*T) {
		temp = particulesTETE->particuleFinale;
		
		/* main gauche */
		if (*MG) {
			temp2 = particulesMG->particuleFinale;
			X = temp[0] - temp2[0];
			Y = temp[1] - temp2[1];
			Z = temp[2] - temp2[2];
			
			if (X*X + Y*Y + Z*Z > 1000000) *MG=false;//1000000 = 1000² 
		}
		/* main droite */
		if (*MD) {
			temp2 = particulesMD->particuleFinale;
			X = temp[0] - temp2[0];
			Y = temp[1] - temp2[1];
			Z = temp[2] - temp2[2];
			
			if (X*X + Y*Y + Z*Z > 1000000) *MD=false;//1000000 = 1000² 
		}
	}
	
	/* évaluation du résultat */
	if (*T) {
		evalFinale[TETE] = eval(particulesTETE->particuleFinale, imgTG,imgTD, TETE);
		if (evalFinale[TETE] < 1.1) *T = false;
		else eval2D->MAJeval();
	}
	if (*MG) {
		evalFinale[MAINgAUCHE] = eval(particulesMG->particuleFinale, imgTG,imgTD, MAINgAUCHE);
		if (evalFinale[MAINgAUCHE] < 1.5) *MG = false;
	}
	if (*MD) {
		evalFinale[MAINdROITE] = eval(particulesMD->particuleFinale, imgTG,imgTD, MAINdROITE);
		if (evalFinale[MAINdROITE] < 1.5) *MD = false;
	}

	/* les cibles ne peuvent pas être au même endrois, on interdit à la réinitialisation les zones déjà sélectionnées *///...devrait être plus générique
	if (*T) {
		if (!*MG) particulesMG->interdire(particulesTETE->particuleFinale, RayonInterdictionT);
		if (!*MD) particulesMD->interdire(particulesTETE->particuleFinale, RayonInterdictionT);
	}
	if (*MG) {
		if (!TETE) particulesTETE->interdire(particulesMG->particuleFinale, RayonInterdictionM);
		if (!*MD) particulesMD->interdire(particulesMG->particuleFinale, RayonInterdictionM);
	}
	if (*MD) {
		if (!TETE) particulesTETE->interdire(particulesMD->particuleFinale, RayonInterdictionM);
		if (!*MG) particulesMG->interdire(particulesMD->particuleFinale, RayonInterdictionM);
	}
	
	/* rééchantillonnage en temps normal, sinon, tentative de réinitialisation douce si çà c'est mal passé *///...devrait être plus générique
	if (!*T) {
		*T = reinit(TETE);
		particulesTETE->selectionneEtReechantillonneApresPerte(*T);
		if (*T) {//si on a retrouvé cette cible, il faut l'interdire à la réinitialisation des autres cibles//...
			if (!*MG) particulesMG->interdire(particulesTETE->particuleFinale, RayonInterdictionT);//...
			if (!*MD) particulesMD->interdire(particulesTETE->particuleFinale, RayonInterdictionT);//...
		}//...
	}
	else particulesTETE->reechantillonne();
	if (!*MG) {
		*MG = reinit(MAINgAUCHE);
		particulesMG->selectionneEtReechantillonneApresPerte(*MG);
		if (*MG) if (!*MD) particulesMD->interdire(particulesMG->particuleFinale, RayonInterdictionM);
	}
	else particulesMG->reechantillonne();
	if (!*MD) {
		*MD = reinit(MAINdROITE);
		particulesMD->selectionneEtReechantillonneApresPerte(*MD);
	}
	else particulesMD->reechantillonne();
	
	/* détermination de l'état complet du système */
	if (*T) {
		if (*MG) {
			if (*MD) ETAT = TOUT;
			else ETAT = PASmAINdROITE;
		} else {
			if (*MD) ETAT = PASmAINgAUCHE;
			else ETAT = AUCUNEmAIN;
		}
	} else {
		if (*MG) {
			if (*MD) ETAT = PAStETE;
			else ETAT = PAStETEnImAINdROITE;
		} else {
			if (*MD) ETAT = PAStETEnImAINgAUCHE;
			else ETAT = RIEN;
		}
	}
	
	/* affichage des particules finales */
	if (AVEC_INTERFACE) {
		if (niveau_reinit[TETE]!=NIVEAU_REINIT_MAX) {
			/************** on reprojette la tête sur les images pour visualiser la particule finale *************/
			int rayon;//rayon du cercle pour l'affichage de la tête (pour simplifier, on néglige les déformations de l'image et on affiche un cercle)
			temp = particulesTETE->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			rayon = int (RayonTete*intrinsicsLeft[2]/temp[2]);
			
			cvCircle( imgG, point2dgauche, rayon, CV_RGB(255,255,255), 2 );//pour simplifier, on néglige les déformations de l'image et on affiche un cercle
			cvCircle( imgD, point2ddroite, rayon, CV_RGB(255,255,255), 2 );
			
			if (!*T) {//si cette cible est perdue, il faut le montrer : on barre la projection
				cvLine( imgG, cvPoint(point2dgauche.x+rayon,point2dgauche.y+rayon), cvPoint(point2dgauche.x-rayon,point2dgauche.y-rayon), CV_RGB(255,255,255), 2);
				cvLine( imgD, cvPoint(point2ddroite.x+rayon,point2ddroite.y+rayon), cvPoint(point2ddroite.x-rayon,point2ddroite.y-rayon), CV_RGB(255,255,255), 2);
			}
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvCircle(imgSup,point2dgauche,rayon,CV_RGB(255,255,255),2);
				else cvCircle(imgSup,point2ddroite,rayon,CV_RGB(255,255,255),2);
			}
		}
		if (niveau_reinit[MAINgAUCHE]!=NIVEAU_REINIT_MAX) {
			/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule gauche finale *************/
			temp = particulesMG->particuleFinale;

			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,255,0),2);
			cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,255,0),2);
			
			if (!*MG) {//si cette cible est perdue, il faut le montrer : on barre la projection
				cvLine( imgG, cvPoint(point2dgauche.x+axesg.width,point2dgauche.y+axesg.height), cvPoint(point2dgauche.x-axesg.width,point2dgauche.y-axesg.height), CV_RGB(255,255,0), 2);
				cvLine( imgD, cvPoint(point2ddroite.x+axesd.width,point2ddroite.y+axesd.height), cvPoint(point2ddroite.x-axesd.width,point2ddroite.y-axesd.height), CV_RGB(255,255,0), 2);
			}
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,angleg,0,360,CV_RGB(255,255,0),2);
				else cvEllipse(imgSup,point2ddroite,axesd,angled,0,360,CV_RGB(255,255,0),2);
			}
		}
		if (niveau_reinit[MAINdROITE]!=NIVEAU_REINIT_MAX) {
			/************** on reprojette l'ellipsoide 3D sur les images pour visualiser la particule droite finale *************/
			temp = particulesMD->particuleFinale;
			
			proj->projettePoint (temp, &point2dgauche, &point2ddroite);//projection du centre
			proj->projectionEllipsoide (temp, projections);
			proj->projetteAxes (projections, &axesg, &axesd);
			angleg = projections[0];
			angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
			
			cvEllipse(imgG,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,255),2);
			cvEllipse(imgD,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,255),2);
			
			if (!*MD) {//si cette cible est perdue, il faut le montrer : on barre la projection
				cvLine( imgG, cvPoint(point2dgauche.x+axesg.width,point2dgauche.y+axesg.height), cvPoint(point2dgauche.x-axesg.width,point2dgauche.y-axesg.height), CV_RGB(255,0,255), 2);
				cvLine( imgD, cvPoint(point2ddroite.x+axesd.width,point2ddroite.y+axesd.height), cvPoint(point2ddroite.x-axesd.width,point2ddroite.y-axesd.height), CV_RGB(255,0,255), 2);
			}
			
			if (AFFICHE_PARTICULES) {//affichage de la particule finale
				if (AFFICHE_SUP_GAUCHE) cvEllipse(imgSup,point2dgauche,axesg,angleg,0,360,CV_RGB(255,0,255),2);
				else cvEllipse(imgSup,point2ddroite,axesd,angled,0,360,CV_RGB(255,0,255),2);
			}
		}
		/************** affichage **************/
		if (AFFICHE_SUP) affiche(imgG, imgD, imgSup);
		else affiche(imgG, imgD);
	}
	
	return ETAT;
}



/*****************************************/
/* fonctions de récupération :           */
/*****************************************/

/* retourne le vecteur d'état global : (TETE,MG,MD)
 * 
 * si l'une des cible n'est plus traquée, c'est son dernier vecteur d'état correct qui est renvoyé
 */
int* Traqueur2::RECUP() {
	
	switch (ETAT) {
		case RIEN:
			memcpy(VEGlobal, particulesTETE->particuleFinalePrec , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinalePrec , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinalePrec , particulesMD->nbvar*sizeof(int));
			break;
		case TOUT:
			memcpy(VEGlobal, particulesTETE->particuleFinale , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinale , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinale , particulesMD->nbvar*sizeof(int));
			break;
		case PASmAINgAUCHE:
			memcpy(VEGlobal, particulesTETE->particuleFinale , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinalePrec , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinale , particulesMD->nbvar*sizeof(int));
			break;
		case PASmAINdROITE:
			memcpy(VEGlobal, particulesTETE->particuleFinale , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinale , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinalePrec , particulesMD->nbvar*sizeof(int));
			break;
		case AUCUNEmAIN:
			memcpy(VEGlobal, particulesTETE->particuleFinale , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinalePrec , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinalePrec , particulesMD->nbvar*sizeof(int));
			break;
		case PAStETE:
			memcpy(VEGlobal, particulesTETE->particuleFinalePrec , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinale , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinale , particulesMD->nbvar*sizeof(int));
			break;
		case PAStETEnImAINgAUCHE:
			memcpy(VEGlobal, particulesTETE->particuleFinalePrec , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinalePrec , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinale , particulesMD->nbvar*sizeof(int));
			break;
		case PAStETEnImAINdROITE:
			memcpy(VEGlobal, particulesTETE->particuleFinalePrec , particulesTETE->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar]), particulesMG->particuleFinale , particulesMG->nbvar*sizeof(int));
			memcpy(&(VEGlobal[particulesTETE->nbvar+particulesMG->nbvar]), particulesMD->particuleFinalePrec , particulesMD->nbvar*sizeof(int));
			break;
	}
	
	return VEGlobal;
}


/* retourne la taille du vecteur d'état : */
int Traqueur2::GETNBVAR() {

	return nbvar;
}


/* si la tête a été détectée ou non ce tour (par la détection de visage != traquée) : */
bool Traqueur2::TETEDETECTEE() {

	if (particulesTETE->nbpartdetectees==0) return false;
	else return true;
}


/*****************************************/
/* fonctions de modification d'obtions : */
/*****************************************/

/*
 * fonctions pour changer la matrice de variance de la dynamique
 * 
 * typeParticule : type de particule qu'on veut affecter par cette modification
 * var : nouvelle matrice de variance (ATTENTION : la taille de la matrice donnée doit correspondre à celle modifiée)
 * 
 * retourne true s'il n'y a pas eu de problème, false autrement
 */
bool Traqueur2::changevar(TYPE_PARTICULE_GEST typeParticule, int *var) {
	
	switch (typeParticule) {
		case TETE:
			return ( particulesTETE->changevar(var) );
		case MAINgAUCHE:
			return ( particulesMG->changevar(var) );
		case MAINdROITE:
			return ( particulesMD->changevar(var) );
		default:
			cout << "traqueur.cpp (changevar) : ERREUR, type de particule (" << typeParticule << ") inexistant" << endl;
			return false;
	}
}


/* si l'on veut utiliser une interface graphique :
 * 
 * oui : true si on veux activer l'interface, false pour la désactiver
 */
void Traqueur2::avec_interface(bool oui) {

	if (oui) {
	
		if (AVEC_INTERFACE) return;
		
		/* création de la fenètre */
		cvNamedWindow("gestimages",1);
		cvMoveWindow("gestimages",0,0);
		
		if (AFFICHE_SUP) img = cvCreateImage(cvSize(w,h*3),8,3);
		else img = cvCreateImage(cvSize(w,h*2),8,3);
		
		/* magouille nécéssaire pour qu'opencv affiche l'image dès le départ */
		cvShowImage("gestimages",img);
		cvWaitKey(100);
	
		AVEC_INTERFACE = true;
		
	} else {
		
		if (!AVEC_INTERFACE) return;
		
		cvDestroyWindow("gestimages");
		cvReleaseImage(&img);
		
		AFFICHE_SUP = false;
		AFFICHE_PARTICULES = false;
		AVEC_INTERFACE = false;
	}
}


/* pour modifier la position de la fenètre :
 * 
 * w,h : position de la fenètre
 */
void Traqueur2::position_interface(int w, int h) {
	
	if (!AVEC_INTERFACE) return;//l'interface doit être activée avant
	cvMoveWindow("gestimages",w,h);
}


/* si l'on veut afficher une image supplémentaire : proba peau, mouvement, distance...
 * 
 * oui : true si on veux afficher une image supplémentaire, false pour désactiver cet affichage
 * gauche : true si l'image supplémentaire doit être basée sur l'image gauche, false pour la droite
 * 
 * Pour afficher cette image supplémentaire, il faut que l'affichage fonctionne déjà!
 */
void Traqueur2::affiche_image_sup(bool oui, bool gauche) {

	if (!AVEC_INTERFACE) return;//l'interface doit être activée avant
	
	if (oui) {
		AFFICHE_SUP_GAUCHE = gauche;
		if (AFFICHE_SUP) return;
		//il faut relancer l'affichage
		avec_interface(false);
		AFFICHE_SUP = true;
		avec_interface(true);
		
	} else {
		
		if (!AFFICHE_SUP) return;
		//il faut relancer l'affichage
		avec_interface(false);
		AFFICHE_SUP = false;
		avec_interface(true);
		
		AFFICHE_PARTICULES = false;
		AFFICHE_PROBA_PEAU = false;
		AFFICHE_MVT = false;
		AFFICHE_DIST = false;
	}
}


/* si l'on veut afficher l'image de proba peau :
 * 
 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
 * 
 * oui : true si on veux afficher cette image, false pour désactiver cette option
 */
void Traqueur2::affiche_proba_peau(bool oui) {
	
	if (!AFFICHE_SUP) return;//l'interface doit être activée avant
	
	if (oui) {
		AFFICHE_PROBA_PEAU = true;
		AFFICHE_MVT = false;
		AFFICHE_DIST = false;
	}
	else AFFICHE_PROBA_PEAU = false;
}
/* si l'on veut afficher l'image de mouvement :
 * 
 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
 * 
 * oui : true si on veux afficher cette image, false pour désactiver cette option
 */
void Traqueur2::affiche_mvt(bool oui) {
	
	if (!AFFICHE_SUP) return;//l'interface doit être activée avant
	
	if (oui) {
		AFFICHE_MVT = true;
		AFFICHE_PROBA_PEAU = false;
		AFFICHE_DIST = false;
	}
	else AFFICHE_MVT = false;
}
/* si l'on veut afficher l'image de distance :
 * 
 * Pour afficher cette image, il faut que l'affichage d'une image supplémentaire fonctionne déjà!
 * 
 * oui : true si on veux afficher cette image, false pour désactiver cette option
 */
void Traqueur2::affiche_dist(bool oui) {
	
	if (!AFFICHE_SUP) return;//l'interface doit être activée avant
	
	if (oui) {
		AFFICHE_DIST = true;
		AFFICHE_PROBA_PEAU = false;
		AFFICHE_MVT = false;
	}
	else AFFICHE_DIST = false;
}


/* si l'on veut afficher les particules :
 * 
 * Pour afficher les particules, il faut que l'affichage de l'image de proba peau fonctionne déjà!
 * 
 * oui : true si on veux afficher les particules, false pour désactiver cette option
 */
void Traqueur2::affiche_particules(bool oui) {
	
	if (!AFFICHE_SUP) return;//l'interface doit être activée avant
	
	if (oui) AFFICHE_PARTICULES = true;
	else AFFICHE_PARTICULES = false;
}


/* si l'on veut sauvegarder les images et/ou les résultats obtenues :
 * 
 * ouiImages : true pour sauvegarder les images, false pour désactiver cette option
 * ouiResultats : true pour sauvegarder les résultats, false pour désactiver cette option
 * cheminSauvegarde : chemin du dossier dans lequel les images seront sauvegardées (NULL pour avoir le chemin par défaut)
 * 
 * renvoie true en temps normal, false s'il y a un problème avec le fichier (mauvais chemin la plupart du temps)
 */
bool Traqueur2::sauvegarde(bool ouiImages, bool ouiResultats, char* cheminSauvegarde) {
	
	if (cheminSauvegarde!=NULL) strcpy(CheminSauvegarde,cheminSauvegarde);
	else strcpy(CheminSauvegarde,CHEMIN_SAUV_DEFAUT);
	
	/* s'il va y avoir sauvegarde, il faut vérifier que le chemin est valide */
	if ((ouiImages)||(ouiResultats)) {
		sprintf(nomsauvegarde,"%s/tmp",CheminSauvegarde);
		fichierResultats = fopen(nomsauvegarde, "w");
		if (!fichierResultats) {//si on n'a pas réussi à créer ce fichier, c'est que le chemin n'existe pas
			printf ("\nERREUR : chemin '%s' non valide!\n",CheminSauvegarde);
			return false;
		} else {//le chemin existe, on supprime le fichier temporaire
			fclose (fichierResultats);
			remove (nomsauvegarde);
			fichierResultats = NULL;
		}
	}
	
	/* fichier de sauvegarde des résultats */
	if (ouiResultats) {
		sprintf(nomsauvegarde,"%sfichierResultat.gest",CheminSauvegarde);
		fichierResultats = fopen(nomsauvegarde, "a");
		if (!fichierResultats) {
			printf("\nERREUR : impossible d'ouvrir ou de créer '%s'\nvérifiez le chemin!\n",nomsauvegarde);
			return false;
		}
		fprintf (fichierResultats, "\ndémarrage de la sauvegarde\n");
	} else if (fichierResultats) {
		fclose(fichierResultats);
		fichierResultats = NULL;
	}
	
	SAUVEGARDE_IMAGES = ouiImages;
	SAUVEGARDE_RESULTATS = ouiResultats;
	return true;
}



/*****************************************/
/* fonctions d'affichage :               */
/*****************************************/

/* enregistre les images données en entrée séparément (et les affiches éventuellement) */
void Traqueur2::enregistreImgs (IplImage* imgg, IplImage* imgd) {
	
	if (SAUVEGARDE_IMAGES) {
		numero++;
		/* image gauche */
		sprintf(nomsauvegarde,"%simage%04d.g.png",CheminSauvegarde,numero);
		cvSaveImage(nomsauvegarde, imgg);
		/* image droite */
		sprintf(nomsauvegarde,"%simage%04d.d.png",CheminSauvegarde,numero);
		cvSaveImage(nomsauvegarde, imgd);
		
		if (SAUVEGARDE_RESULTATS) ecritResultats();//pour récupérer les données de pom (même s'il n'y a pas de traque)
	}
	else if (SAUVEGARDE_RESULTATS) { numero++; ecritResultats(); }
	
	if (AVEC_INTERFACE) {
		memcpy(img->imageData, imgg->imageData, w*h*3*sizeof(char));
		memcpy(img->imageData + (w*h*3*sizeof(char)), imgd->imageData, w*h*3*sizeof(char));
		
		cvShowImage("gestimages",img);
		cvWaitKey(2);
	}
}


/* affichage sur l'écran : 2 images l'une en dessous de l'autre */
void Traqueur2::affiche (IplImage* imgg, IplImage* imgd) {

	if (!AVEC_INTERFACE) return;

	memcpy(img->imageData, imgg->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (w*h*3*sizeof(char)), imgd->imageData, w*h*3*sizeof(char));
	
	cvShowImage("gestimages",img);
	cvWaitKey(2);
	
	if (SAUVEGARDE_IMAGES) {
		numero++;
		sprintf(nomsauvegarde,"%simage%04d.png",CheminSauvegarde,numero);
		cvSaveImage(nomsauvegarde, img);
		if (SAUVEGARDE_RESULTATS) ecritResultats();
	}
	else if (SAUVEGARDE_RESULTATS) { numero++; ecritResultats(); }
}


/* affichage sur l'écran : 3 images l'une en dessous de l'autre */
void Traqueur2::affiche (IplImage* imgg, IplImage* imgd, IplImage* img3) {

	if (!AVEC_INTERFACE) return;
	if (!AFFICHE_SUP) return;
	
	memcpy(img->imageData, imgg->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (w*h*3*sizeof(char)), imgd->imageData, w*h*3*sizeof(char));
	memcpy(img->imageData + (2*w*h*3*sizeof(char)), img3->imageData, w*h*3*sizeof(char));
	
	cvShowImage("gestimages",img);
	cvWaitKey(2);
	
	if (SAUVEGARDE_IMAGES) {
		numero++;
		sprintf(nomsauvegarde,"%simage%04d.png",CheminSauvegarde,numero);
		cvSaveImage(nomsauvegarde, img);
		if (SAUVEGARDE_RESULTATS) ecritResultats();
	}
	else if (SAUVEGARDE_RESULTATS) { numero++; ecritResultats(); }
}



/**********************/
/* fonctions internes */
/**********************/

/* calcule de la zone maximum (i.e. d'initialisation) de recherche d'une cible :
 * les zones de recherche sont stoquées sous forme de leurs centres et taille de demi-axes (contenus respectivement dans centres et zones3D)
 * 
 * type : cible pour laquelle calculer la zone
 * 
 * retourne true si tout s'est bien passée, false autrement
 */
bool Traqueur2::calculeZoneMax(TYPE_PARTICULE_GEST type) {//... ici, on pourrait peut-être améliorer les choses grâce à humpos : si on a les pieds, on sait si les mains qu'on a vu sont devant ou derrière l'homme, par ex.
	//... il faudrait que la définition des zones soit stoquée sous une autre forme (variables globales? fichier de configuration?)
	switch (type) {
		case TETE:
			
			if (ETAT==PAStETE) {//on part des deux mains
				centres[type][0] = int ((particulesMD->particuleFinale[0]+particulesMG->particuleFinale[0])/2);
				centres[type][1] = int ((particulesMD->particuleFinale[1]+particulesMG->particuleFinale[1])/2);
				centres[type][2] = int ((particulesMD->particuleFinale[2]+particulesMG->particuleFinale[2])/2);
				zones3D[type][0] = 500+abs(particulesMD->particuleFinale[0]-centres[type][0]);//0.5m + dist MG/MDm + 0.5m en largeur
				zones3D[type][1] = 1000;//1m+1m en hauteur (en haut et en bas du milieu des deux mains)
				zones3D[type][2] = 1000;//1m+1m en profondeur (devant et derrière le milieu des deux mains)
				return true;
			}
			if (ETAT==PAStETEnImAINgAUCHE) {//on part de la main droite
				centres[type][0] = particulesMD->particuleFinale[0]+250;
				centres[type][1] = particulesMD->particuleFinale[1];
				centres[type][2] = particulesMD->particuleFinale[2];
				zones3D[type][0] = 750;//0.5m+1m en largeur (à droite et à gauche de la main droite)
				zones3D[type][1] = 1000;//1m+1m en hauteur (en haut et en bas de la main droite)//...là il faudra enlever ce qui est impossible (à partir de la hauteur de la main, par rapport à une taille min et max d'un être humain)
				zones3D[type][2] = 1000;//1m+1m en profondeur (devant et derrière de la main droite)//...là il faudra enlever ce qui est impossible (à partir de la position de humpos ?)
				return true;
			}
			if (ETAT==PAStETEnImAINdROITE) {//on part de la main gauche
				centres[type][0] = particulesMG->particuleFinale[0]-250;
				centres[type][1] = particulesMG->particuleFinale[1];
				centres[type][2] = particulesMG->particuleFinale[2];
				zones3D[type][0] = 750;//0.5m+1m en largeur (à gauche et à droite de la main gauche)
				zones3D[type][1] = 1000;//1m+1m en hauteur (en haut et en bas de la main gauche)//...là il faudra enlever ce qui est impossible (à partir de la hauteur de la main, par rapport à une taille min et max d'un être humain)
				zones3D[type][2] = 1000;//1m+1m en profondeur (devant et derrière de la main gauche)//...là il faudra enlever ce qui est impossible (à partir de la position de humpos ?)
				return true;
			}
			break;
			
		case MAINgAUCHE:
			
			if ((ETAT==PASmAINgAUCHE)||(ETAT==AUCUNEmAIN)) {//on peut tout faire à partir de la tête
				centres[type][0] = particulesTETE->particuleFinale[0]+600;//on laisse 10 cm de plus pour éviter de chopper la tête
				centres[type][1] = particulesTETE->particuleFinale[1];
				centres[type][2] = particulesTETE->particuleFinale[2];
				zones3D[type][0] = 500;//1m en largeur (côté droit)
				zones3D[type][1] = 1000;//1m+1m en hauteur (en haut et en bas de la tête)
				zones3D[type][2] = 1000;//1m+1m en profondeur (devant et derrière de la tête)
				return true;
			}
			if (ETAT==PAStETEnImAINgAUCHE) {//on part de la main droite
				centres[type][0] = particulesMD->particuleFinale[0]+750;
				centres[type][1] = particulesMD->particuleFinale[1];
				centres[type][2] = particulesMD->particuleFinale[2];
				zones3D[type][0] = 1250;//0.5m+2m en largeur (à droite et à gauche de la main droite)
				zones3D[type][1] = 2000;//2m+2m en hauteur (en haut et en bas de la main droite)//...là il faudra enlever ce qui est impossible (à partir de la hauteur de la main, par rapport à une taille min et max d'un être humain)
				zones3D[type][2] = 2000;//2m+2m en profondeur (devant et derrière de la main droite)//...là il faudra enlever ce qui est impossible (à partir de la position de humpos ?)
				return true;
			}
			break;
			
		case MAINdROITE:
			
			if ((ETAT==PASmAINdROITE)||(ETAT==AUCUNEmAIN)) {//on peut tout faire à partir de la tête
				centres[type][0] = particulesTETE->particuleFinale[0]-600;//on laisse 10 cm de plus pour éviter de chopper la tête
				centres[type][1] = particulesTETE->particuleFinale[1];
				centres[type][2] = particulesTETE->particuleFinale[2];
				zones3D[type][0] = 500;//1m en largeur (côté droit)
				zones3D[type][1] = 1000;//1m+1m en hauteur (en haut et en bas de la tête)
				zones3D[type][2] = 1000;//1m+1m en profondeur (devant et derrière de la tête)
				return true;
			}
			if (ETAT==PAStETEnImAINdROITE) {//on part de la main gauche
				centres[type][0] = particulesMG->particuleFinale[0]-750;
				centres[type][1] = particulesMG->particuleFinale[1];
				centres[type][2] = particulesMG->particuleFinale[2];
				zones3D[type][0] = 1250;//0.5m+2m en largeur (à gauche et à droite de la main gauche)
				zones3D[type][1] = 2000;//2m+2m en hauteur (en haut et en bas de la main gauche)//...là il faudra enlever ce qui est impossible (à partir de la hauteur de la main, par rapport à une taille min et max d'un être humain)
				zones3D[type][2] = 2000;//2m+2m en profondeur (devant et derrière de la main gauche)//...là il faudra enlever ce qui est impossible (à partir de la position de humpos ?)
				return true;
			}
			break;
	}
	
	/* si on arrive ici, c'est qu'il y a eu un problème */
	printf("ERREUR calculeZoneMax : cette zone ne devrait pas être calculée de cette manière!\n");
	printf("type = %d , ETAT = %d\n",type,ETAT);
	return false;
}


/* fonction de réinitialisation du traqueur : 
 * 
 * type : type de particule à réinitialiser
 * 
 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
 * Aucune nouvelle détection n'est faite ici.
 * 
 * retourne true si la réinitialisation s'est bien passée, false autrement
 * on réactualise également le niveau de réinitialisation
 */
bool Traqueur2::reinit(TYPE_PARTICULE_GEST type) {
	
	int xmin,xmax,ymin,ymax,zmin,zmax, X,Y,Z;
	int nbvarcour;//nombre de variables du type de particules à réinitialiser
	int incertitude = 200;//incertitude en z
	
	if (BAVARD) cout << "tentative de REINITIALISATION douce de ";
	
	switch (type) {
		
		case TETE:
			particuleTemp = particulesTETE;
			nbvarcour = particulesTETE->nbvar;
			if (BAVARD) cout << "la TETE ";
			break;
		case MAINgAUCHE:
			particuleTemp = particulesMG;
			nbvarcour = particulesMG->nbvar;
			if (BAVARD) cout << "la MAIN GAUCHE ";
			break;
		case MAINdROITE:
			particuleTemp = particulesMD;
			nbvarcour = particulesMD->nbvar;
			if (BAVARD) cout << "la MAIN DROITE ";
			break;
		default:
			cout << "ERREUR : type de particule inexistant : erreur de programmation!!\n";
			return false;
	}
	
	/* centre de la zone */
	if (niveau_reinit[type] == NIVEAU_REINIT_MAX) {
		if (BAVARD) cout << "(réinitialisation totale)" << endl;
		X=centres[type][0];
		Y=centres[type][1];
		Z=centres[type][2];
	} else {
		if (particuleTemp->selectionFaite) {
			if (BAVARD) printf("(cause : la particule finale était très mauvaise (%1.2f))\n",evalFinale[type]);
			X=particuleTemp->particuleFinalePrec[0];
			Y=particuleTemp->particuleFinalePrec[1];
			Z=particuleTemp->particuleFinalePrec[2];
		} else {
			if (BAVARD) cout << "(cause : toutes les évaluations étaient a 0)" << endl;//particuleFinale contient donc encore la particuleFinale du pas précédent
			X=particuleTemp->particuleFinale[0];
			Y=particuleTemp->particuleFinale[1];
			Z=particuleTemp->particuleFinale[2];
		}
	}
	
	if (particuleTemp->nbpartdetectees==0) {
		if (BAVARD) cout << "Aucune détection, réinitialisation annulée" << endl;
		if (niveau_reinit[type]!=NIVEAU_REINIT_MAX) niveau_reinit[type]++;
		return false;
	}
	
	/* taille de la zone */
	xmin=X-zones3D[type][0];
	xmax=X+zones3D[type][0];
	ymin=Y-zones3D[type][1];
	ymax=Y+zones3D[type][1];
	zmin=Z-zones3D[type][2];
	zmax=Z+zones3D[type][2];
	
	/* on cherche une particule pas trop éloignée de la particule du tour précédent */
	int i=-1;
	int distcarre;//distance entre la particule finale précédente et la détection en cours
	int distmincarre = 100000000;//distance min entre la particule finale précédente et la détection
	for (int j=0;j<particuleTemp->nbpartdetectees;j++) {
		if ((particuleTemp->particuleDetectees[j][0] < xmax)&&(particuleTemp->particuleDetectees[j][0] > xmin)) {
			distcarre = (particuleTemp->particuleDetectees[j][0] - X)*(particuleTemp->particuleDetectees[j][0] - X);
			if ((particuleTemp->particuleDetectees[j][1] < ymax)&&(particuleTemp->particuleDetectees[j][1] > ymin)) {
				distcarre += (particuleTemp->particuleDetectees[j][1] - Y)*(particuleTemp->particuleDetectees[j][1] - Y);
				if ((particuleTemp->particuleDetectees[j][2] < zmax)&&(particuleTemp->particuleDetectees[j][2] > zmin)) {
					distcarre += (particuleTemp->particuleDetectees[j][2] - Z)*(particuleTemp->particuleDetectees[j][2] - Z);
					if (distcarre<distmincarre) if (!particuleTemp->detecteeinterdit(j)) { i=j; distmincarre = distcarre; }//choix de l'ellipsoide, on prend la plus proche de la position précédente (qui ne soit pas interdite)
				}
			}
		}
	}
	
	if (i==-1) {
		if (BAVARD) cout << "Aucune détection correspondante dans la zone de recherche, réinitialisation annulée" << endl;
		if (niveau_reinit[type]!=NIVEAU_REINIT_MAX) niveau_reinit[type]++;
		return false;
	}
	
	/* vérifications : une main ne doit pas se trouver à plus de 1m de la tête */
	switch (type) {
		
		case TETE:
			break;
			
		case MAINgAUCHE:
			if (*T) {
				temp = particulesTETE->particuleFinale;
				temp2 = particuleTemp->particuleDetectees[i];
				X = temp[0] - temp2[0];
				Y = temp[1] - temp2[1];
				Z = temp[2] - temp2[2];
				Z = (Z>incertitude)?(Z-incertitude):0;
				if (X*X + Y*Y + Z*Z > 1000000) {//1000000 = 1000²
					if (BAVARD) printf("réinitialisation annulée : trop loin de la tête\n");
					if (niveau_reinit[type]!=NIVEAU_REINIT_MAX) niveau_reinit[type]++;
					return false;
				}
			}
			break;
			
		case MAINdROITE:
			if (*T) {//une main ne doit pas se trouver à plus de 1m de la tête
				temp = particulesTETE->particuleFinale;
				temp2 = particuleTemp->particuleDetectees[i];
				X = temp[0] - temp2[0];
				Y = temp[1] - temp2[1];
				Z = temp[2] - temp2[2];
				Z = (Z>incertitude)?(Z-incertitude):0;
				if (X*X + Y*Y + Z*Z > 1000000) {//1000000 = 1000²
					if (BAVARD) printf("réinitialisation annulée : trop loin de la tête\n");
					if (niveau_reinit[type]!=NIVEAU_REINIT_MAX) niveau_reinit[type]++;
					return false;
				}
			}
			break;
	}
	
	memcpy(particuleTemp->particuleFinale, particuleTemp->particuleDetectees[i], nbvarcour*sizeof(int));
	
	if (BAVARD) cout << "réinitialisation douce faite" << endl;
	
	/* la particule pseudo-initiale est l'équivalent d'une particule parfaite (poids 1, aucune autre particule conservée) */
	memcpy(particuleTemp->particulesPrec[0],particuleTemp->particuleFinale,nbvarcour*sizeof(int));
	particuleTemp->nbpartconservees = 1;
	particuleTemp->poidsPrec[0] = 1;
	
	niveau_reinit[type]=0;
	return true;
}


/* fonction évaluant une particule via les fonctions d'évaluation 2D :
 * 
 * particule : la particule à évaluer
 * imgG, imgD : images en cours
 * type : type de particule à réinitialiser (et donc type d'évaluation à utiliser)
 * 
 * renvoie son évaluation (entre 0 et 1)
 */
double Traqueur2::eval(int* particule, IplImage* imgG, IplImage* imgD, TYPE_PARTICULE_GEST type) {
	
	double eval;
	int rayon;
	
	proj->projettePoint (particule, &point2dgauche, &point2ddroite);//projection du centre
	
	switch (type) {
	
	case TETE:
		rayon = int (RayonTete*intrinsicsLeft[2]/particule[2]);
		
		if (CalculeImgDist) {
			eval = eval2D->evalCercle(detecteurBlobTETE->getImgProbaPeauG(), imgG, mesureDistance->getImgDistG(), point2dgauche, rayon);
			eval *= eval2D->evalCercle(detecteurBlobTETE->getImgProbaPeauD(), imgD, mesureDistance->getImgDistD(), point2ddroite, rayon);
		} else {
			eval = eval2D->evalCercle(detecteurBlobTETE->getImgProbaPeauG(), imgG, NULL, point2dgauche, rayon);
			eval *= eval2D->evalCercle(detecteurBlobTETE->getImgProbaPeauD(), imgD, NULL, point2ddroite, rayon);
		}
		/* mesure de distance 3D entre la particule et le visage détecté */
		if (particulesTETE->nbpartdetectees!=0) {//si on a détecté un visage (on considère qu'on en a toujours qu'un seul)//...
			
			temp = particulesTETE->particuleDetectees[0];
			
			int X = temp[0] - particule[0];
			int Y = temp[1] - particule[1];
			int Z = temp[2] - particule[2];
			
			int distcarre = X*X + Y*Y + Z*Z;
			
			eval *= exp(-distcarre/sigmavisage);
		}
		
		break;
	
	case MAINgAUCHE:
		proj->projettePoint (particule, &point2dgauche, &point2ddroite);//projection du centre
		proj->projectionEllipsoide (particule, projections);
		proj->projetteAxes (projections, &axesg, &axesd);
		angleg = projections[0];
		angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
		if (CalculeImgDist) {
			eval = eval2D->evalEllipse(detecteurBlob->getImgProbaPeauG(), mesureMvt->getImgMvtG(), mesureDistance->getImgDistG(), point2dgauche, axesg, angleg);
			eval *= eval2D->evalEllipse(detecteurBlob->getImgProbaPeauD(), mesureMvt->getImgMvtD(), mesureDistance->getImgDistD(), point2ddroite, axesd, angled);
		} else {
			eval = eval2D->evalEllipse(detecteurBlob->getImgProbaPeauG(), mesureMvt->getImgMvtG(), NULL, point2dgauche, axesg, angleg);
			eval *= eval2D->evalEllipse(detecteurBlob->getImgProbaPeauD(), mesureMvt->getImgMvtD(), NULL, point2ddroite, axesd, angled);
		}
		
		break;
		
	case MAINdROITE:
		proj->projettePoint (particule, &point2dgauche, &point2ddroite);//projection du centre
		proj->projectionEllipsoide (particule, projections);
		proj->projetteAxes (projections, &axesg, &axesd);
		angleg = projections[0];
		angled = angleg;//c'est approximatif, mais ça suffit largement et fait gagner du temps
		
		if (CalculeImgDist) {
			eval = eval2D->evalEllipse(detecteurBlob->getImgProbaPeauG(), mesureMvt->getImgMvtG(), mesureDistance->getImgDistG(), point2dgauche, axesg, angleg);
			eval *= eval2D->evalEllipse(detecteurBlob->getImgProbaPeauD(), mesureMvt->getImgMvtD(), mesureDistance->getImgDistD(), point2ddroite, axesd, angled);
		} else {
			eval = eval2D->evalEllipse(detecteurBlob->getImgProbaPeauG(), mesureMvt->getImgMvtG(), NULL, point2dgauche, axesg, angleg);
			eval *= eval2D->evalEllipse(detecteurBlob->getImgProbaPeauD(), mesureMvt->getImgMvtD(), NULL, point2ddroite, axesd, angled);
		}
		
		break;
		
	default:
		cout << "ERREUR : type de particule inexistant : erreur de programmation!!\n";
		return false;
	}
	
	return eval;
}


/* fonction écrivant les résultats du traqueur dans le fichier de résultats courant :
 * 
 */
void Traqueur2::ecritResultats() {
	
	int i;
	struct timeval tv;
	
	fprintf (fichierResultats, "\ntour numéro %d\n",numero);
	gettimeofday(&tv, NULL);
	fprintf (fichierResultats, "temps cpu : %lld\n", (tv.tv_sec * (unsigned long long int)1000000L + tv.tv_usec)/1000 );
	fprintf (fichierResultats, "état du suivi : %d\n",(int)ETAT);
	/* 
	 * 	avec ETAT :
	 * 	RIEN=0,			//rien n'est traqué
	 * 	TOUT=1,			//tout est traqué
	 * 	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
	 * 	PASmAINdROITE=3,	//tout est traqué sauf la main droite
	 * 	AUCUNEmAIN=4,		//seule la tête est traquée
	 * 	PAStETE=5,		//tout est traqué sauf la tête
	 * 	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
	 * 	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
	 */
	
	fprintf (fichierResultats, "vecteur d'état (de taille %d) :\n",GETNBVAR());
	fprintf (fichierResultats, "ATTENTION : on utilise ici les conventions vision != des conventions du poster!!!\n");
	
	RECUP();
	for (i=0; i<GETNBVAR(); i++) fprintf (fichierResultats, "%d\n",VEGlobal[i]);
	
	if (TETEDETECTEE()) fprintf (fichierResultats, "Tête détectée\n");
	else fprintf (fichierResultats, "Tête NON détectée\n");
	
	fprintf (fichierResultats, "recup Pompos : (ordre : x,y,z,roll,pitch,yaw)\n");
	fprintf (fichierResultats, "sensorToMain :\n");
	for (i=0; i<6; i++) fprintf (fichierResultats, "%4.3f\n",recupPompos.sensorToMain[i]);
	fprintf (fichierResultats, "mainToOrigin :\n");
	for (i=0; i<6; i++) fprintf (fichierResultats, "%4.3f\n",recupPompos.mainToOrigin[i]);
}

/* fonction effectuant les vérifications qu'opencv ne gère pas sur les rectangles :
 * pour définir une ROI comme pour dessiner un rectangle sur une image, il faut :  rect.w et rect.h >= 5  ,  0 <= rect.x et rect.y < w et h
 * 
 * rect : le rectangle à vérifier
 */
void Traqueur2::verifRectOpencv(CvRect& rect) {
	
	//il faut rect.x et rect.y >= 0 et < w et h
	if (rect.x < 0) {
		rect.width += rect.x;//on va forcer x à 0, la largeur doit donc être emputée du dépassement
		rect.x = 0;
	}
	if (rect.y < 0) {
		rect.height += rect.y;//on va forcer y à 0, la hauteur doit donc être emputée du dépassement
		rect.y = 0;
	}
	//pour x ou y trop grand, pas besoin de rectifier w : de toutes façon, c'est en dehors de l'image
	if (rect.x >= w) rect.x = w-1;
	if (rect.y >= h) rect.y = h-1;
	//il faut rect.w et rect.h >= 5
	if (rect.width < 5) rect.width = 5;
	if (rect.height < 5) rect.height = 5;
}
